library(SingleCellExperiment)
library(tidyverse)
library(ggpubr)
library(cowplot)
library(ggrepel)
library(ggnewscale)
library(ggh4x)
library(mgcViz)
library(scDesign3)
library(scran)
library(scuttle)
library(rvinecopulib)
library(biomaRt)
theme_set(theme_bw())

impute_dir = "/home/qingyang/SpatialBenchmarking/Dataset1_res/"
csv_data = c("Tangram", "stPlus", "gimVI", "novoSpaRc", "SpaGE", "SpaOTsc")
txt_data = c("Seurat","LIGER")
location = read.table("/home/qingyang/SpatialBenchmarking/Dataset1/Locations.txt",header = TRUE)
genes = read.csv("/home/qingyang/SpatialBenchmarking/Dataset1/test_gene.csv")
data = c(csv_data,txt_data)
#data = c("Tangram", "stPlus", "gimVI")
location = as.data.frame(location)
colnames(location) = c("x","y")
sce_list = lapply(data, function(x){
  print(x)
  cur_path = paste0(impute_dir,x)
  if(x %in% csv_data){
    count_mat = t(read.csv(paste0(cur_path,"_impute.csv"), row.names = "X"))
    rownames(count_mat) <- stringr::str_to_title(rownames(count_mat))
    count_mat <- count_mat[genes$test_gene, ]
    rownames(location) = colnames(count_mat)
    sce = SingleCellExperiment(list(counts = count_mat), colData = location)
  }else{
    count_mat = read.table(paste0(cur_path,"_impute.txt"))
    count_mat <- count_mat[genes$test_gene, ]
    rownames(location) = colnames(count_mat)
    sce = SingleCellExperiment(list(counts = count_mat), colData = location)
  }
  
  return(sce)
})
names(sce_list) = data

saveRDS(sce_list, "/home/dongyuan/scDesign3/data/SpatialBenchmarking/Dataset1_res/sce_list.rds")

scDesign3_res <- lapply(1:length(data), function(y){
  set.seed(123)
  sce = sce_list[[y]]
  print(data[y])
  print(Sys.time())
  simu <- scdesign3(
    sce = sce,
    assay_use = "counts",
    celltype = NULL,
    pseudotime = NULL,
    spatial = c("x", "y"),
    other_covariates = NULL,
    mu_formula = "s(x, y, bs = 'gp', k = 400, m = c(1, 0.5, 1))",
    sigma_formula = "1",
    family_use = "nb",
    n_cores = 50,
    usebam = FALSE,
    corr_formula = "1",
    copula = "gaussian",
    parallelization = "pbmcmapply", 
    return_model = TRUE
    
  )
  #path_save = paste0(impute_dir,data[y])
  #saveRDS(simu, paste0(path_save,"_scDesign3.rds"))
  return(simu)
})
names(scDesign3_res) = data

saveRDS(scDesign3_res, "/home/dongyuan/scDesign3/data/SpatialBenchmarking/Dataset1_res/all_scDesign3_final.rds")
print("done")


all_scDesign3 = readRDS("/home/dongyuan/scDesign3/data/SpatialBenchmarking/Dataset1_res/all_scDesign3_final.rds")
bic = sapply(all_scDesign3, function(x){
  x$model_bic
})
colnames(bic) = data
saveRDS(bic, "/home/dongyuan/scDesign3/data/SpatialBenchmarking/Dataset1_res/bic.rds")

## ground truth
dat = read.table("/home/qingyang/SpatialBenchmarking/Dataset1/Insitu_count.txt", header = TRUE)
genes = read.csv("/home/qingyang/SpatialBenchmarking/Dataset1/test_gene.csv")
dat = dat[,genes$test_gene]
rownames(location) = rownames(dat)
sce = SingleCellExperiment(list(counts = t(dat)), colData = location)
saveRDS(sce, "/home/dongyuan/scDesign3/data/SpatialBenchmarking/Dataset1_res/sce_true.rds")
print(Sys.time())
set.seed(123)
simu <- scdesign3(
  sce = sce,
  assay_use = "counts",
  celltype = NULL,
  pseudotime = NULL,
  spatial = c("x", "y"),
  other_covariates = NULL,
  mu_formula = "s(x, y, bs = 'gp', k = 400, m = c(1, 0.5, 1))",
  sigma_formula = "1",
  family_use = "nb",
  n_cores = 50,
  usebam = FALSE,
  corr_formula = "1",
  copula = "gaussian",
  parallelization = "pbmcmapply", 
  return_model = TRUE
)
saveRDS(simu, "/home/dongyuan/scDesign3/data/SpatialBenchmarking/Dataset1_res/truth_scdesign3_final.rds")
print("done")