library(scDesign3)
library(scater)
library(scran)


ATAC_sce <- readRDS("~/scDesign3/data/sce_hvg/ATAC_sce.rds")
logcounts(ATAC_sce) <- log1p(counts(ATAC_sce))

ncell <- dim(ATAC_sce)[2]

ntrain <- round(ncell/2)

set.seed(123)
train_index <- sample(seq_len(ncell), ntrain, replace = FALSE)

train_sce <- ATAC_sce[, train_index]
test_sce <- ATAC_sce[, -train_index]
saveRDS(train_sce, "~/scDesign3/data/paper_writting/ATAC/train_sce.rds")
saveRDS(test_sce, "~/scDesign3/data/paper_writting/ATAC/test_sce.rds")

### Construct Formula Matrix
const_mu_formula <- "1"
discrete_mu_formula <- "cell_type"
continuous_mu_formula <- "s(pseudotime, k = 10, bs = 'cr')"

mu_formula <- as.factor(c(const_mu_formula, discrete_mu_formula, continuous_mu_formula))

const_sigma_formula <- "1"
discrete_sigma_formula <- "cell_type"
continuous_sigma_formula <- "s(pseudotime, k = 3, bs = 'cr')"

sigma_formula <- as.factor(c(const_sigma_formula, discrete_sigma_formula, continuous_sigma_formula))

ind_cor_formula <- "ind"
const_cor_formula <- "1"
discrete_cor_formula <- "cell_type"
continuous_cor_formula <- "pseudotime"

corr_formula <- as.factor(c(ind_cor_formula, const_cor_formula, discrete_cor_formula, continuous_cor_formula))


formula_mat <- expand.grid(mu_formula = mu_formula, sigma_formula = sigma_formula, corr_formula = corr_formula)
use_index <- c(1, 10, 2, 11, 20, 5, 14, 23)
formula_mat_ATAC <- formula_mat[use_index, ]
saveRDS(formula_mat_ATAC, "~/scDesign3/data/paper_writting/ATAC/formula_mat.rds")


scDesign3VineZIP_result <- apply(formula_mat_ATAC, 1, function(x) {
  print(x)
  start <- Sys.time()
  res <- scDesign3::scdesign3(sce = train_sce,
                              assay_use = "counts",
                              pseudotime = NULL,
                              celltype = "cell_type",
                              spatial = NULL,
                              other_covariates = NULL,
                              mu_formula = x[1],
                              sigma_formula = x[2],
                              cor_formula = x[3],
                              copula = "vine",
                              usebam = FALSE,
                              n_cores = 24,
                              family_set = c("gauss"),
                              family = "zip")
  end <- Sys.time()
  print(end - start)
  return(res)
}
)

saveRDS(scDesign3VineZIP_result, "~/scDesign3/data/paper_writting/ATAC/scDesign3VineZIP_result.rds")

scDesign3VineNB_result <- apply(formula_mat_ATAC, 1, function(x) {
  print(x)
  start <- Sys.time()
  res <- scDesign3::scdesign3(sce = train_sce,
                              assay_use = "counts",
                              pseudotime = NULL,
                              celltype = "cell_type",
                              spatial = NULL,
                              other_covariates = NULL,
                              mu_formula = x[1],
                              sigma_formula = x[2],
                              cor_formula = x[3],
                              copula = "vine",
                              usebam = FALSE,
                              n_cores = 24,
                              family_set = c("gauss"),
                              family = "nb")
  end <- Sys.time()
  print(end - start)
  return(res)
}
)

saveRDS(scDesign3VineNB_result, "~/scDesign3/data/paper_writting/ATAC/scDesign3VineNB_result.rds")