library(scDesign3)
library(scater)
library(scran)


BATCH_sce <- readRDS("~/scDesign3/data/sce_hvg/BATCH_sce.rds")
logcounts(BATCH_sce) <- log1p(counts(BATCH_sce))

ncell <- dim(BATCH_sce)[2]

ntrain <- round(ncell/2)

set.seed(123)
train_index <- sample(seq_len(ncell), ntrain, replace = FALSE)

train_sce <- BATCH_sce[, train_index]
test_sce <- BATCH_sce[, -train_index]
saveRDS(train_sce, "~/scDesign3/data/paper_writting/BATCH/train_sce.rds")
saveRDS(test_sce, "~/scDesign3/data/paper_writting/BATCH/test_sce.rds")

### Construct Formula Matrix
const_mu_formula <- "1"
discrete_mu_formula <- "cell_type"
continuous_mu_formula <- "cell_type + batch"

mu_formula <- as.factor(c(const_mu_formula, discrete_mu_formula, continuous_mu_formula))

const_sigma_formula <- "1"
discrete_sigma_formula <- "cell_type"
continuous_sigma_formula <- "cell_type + batch"

sigma_formula <- as.factor(c(const_sigma_formula, discrete_sigma_formula, continuous_sigma_formula))

ind_cor_formula <- "ind"
const_cor_formula <- "1"
discrete_cor_formula <- "cell_type"
continuous_cor_formula <- "cell_type + batch"

corr_formula <- as.factor(c(ind_cor_formula, const_cor_formula, discrete_cor_formula, continuous_cor_formula))


formula_mat <- expand.grid(mu_formula = mu_formula, sigma_formula = sigma_formula, corr_formula = corr_formula)
use_index <- c(1, 10, 2, 11, 20, 5, 14, 23, 3, 12, 9, 18)
formula_mat_BATCH <- formula_mat[use_index, ]

saveRDS(formula_mat_BATCH, "~/scDesign3/data/paper_writting/BATCH/formula_mat.rds")


scDesign3Vine_result <- apply(formula_mat_BATCH, 1, function(x) {
  print(x)
  start <- Sys.time()
  if(x[3] == "cell_type + batch") x_3 = c("cell_type", "batch") 
  else x_3= x[3]
  res <- scDesign3::scdesign3(sce = train_sce,
                              assay_use = "counts",
                              pseudotime = NULL,
                              celltype = "cell_type",
                              spatial = NULL,
                              other_covariates = "batch",
                              mu_formula = x[1],
                              sigma_formula = x[2],
                              cor_formula = x_3,
                              copula = "vine",
                              usebam = FALSE,
                              n_cores = 24,
                              family_set = c("gauss"),
                              family = "nb")
  end <- Sys.time()
  print(end - start)
  return(res)
}
)

saveRDS(scDesign3Vine_result, "~/scDesign3/data/paper_writting/BATCH/scDesign3Vine_result.rds")