library(scDesign3)
library(scater)
library(scran)
library(tidyverse)

MARROW_sce <- readRDS("~/scDesign3/data/sce_hvg/MARROW_sce.rds")
logcounts(MARROW_sce) <- log1p(counts(MARROW_sce))

ncell <- dim(MARROW_sce)[2]

ntrain <- round(ncell/2)

set.seed(123)
train_index <- sample(seq_len(ncell), ntrain, replace = FALSE)

train_sce <- MARROW_sce[, train_index]
test_sce <- MARROW_sce[, -train_index]
saveRDS(train_sce, "~/scDesign3/data/paper_writting/MARROW/train_sce.rds")
saveRDS(test_sce, "~/scDesign3/data/paper_writting/MARROW/test_sce.rds")

### Construct Formula Matrix
const_mu_formula <- "1"
discrete_mu_formula <- "cell_type"
continuous_mu_formula <- "s(pseudotime1, k = 10, by = l1, bs = 'cr') + s(pseudotime2, k = 10, by = l2, bs = 'cr')"

mu_formula <- as.factor(c(const_mu_formula, discrete_mu_formula, continuous_mu_formula))

const_sigma_formula <- "1"
discrete_sigma_formula <- "cell_type"
continuous_sigma_formula <- "s(pseudotime1, k = 3, by = l1, bs = 'cr') + s(pseudotime2, k = 3, by = l2, bs = 'cr')"

sigma_formula <- as.factor(c(const_sigma_formula, discrete_sigma_formula, continuous_sigma_formula))

ind_cor_formula <- "ind"
const_cor_formula <- "1"
discrete_cor_formula <- "cell_type"
continuous_cor_formula <- "pseudotime"

corr_formula <- as.factor(c(ind_cor_formula, const_cor_formula, discrete_cor_formula, continuous_cor_formula))

formula_mat <- expand.grid(mu_formua = mu_formula, sigma_formula = sigma_formula, corr_formula = corr_formula)
use_index <- c(1, 10, 2, 11, 20, 5, 14, 23, 3, 12, 30, 9, 18, 36)
formula_mat_MARROW <- as_tibble(formula_mat[use_index, ])

#formula_mat_MARROW <- formula_mat_MARROW %>% dplyr::mutate(corr_formula = if_else(corr_formula == "pseudotime", true = c("pseudotime1", "pseudotime2", "l1", "l2"), false = corr_formula)) 

saveRDS(formula_mat_MARROW, "~/scDesign3/data/paper_writting/MARROW/formula_mat.rds")


scDesign3Vine_result <- apply(formula_mat_MARROW, 1, function(x) {
  print(x)
  start <- Sys.time()
  
  if(x[3] == "pseudotime") x_3 = c("pseudotime1", "pseudotime2") 
  else x_3= x[3]
  res <- scDesign3::scdesign3(sce = train_sce,
                              assay_use = "counts",
                              pseudotime = c("pseudotime1", "pseudotime2", "l1", "l2"),
                              celltype = "cell_type",
                              spatial = NULL,
                              other_covariates = NULL,
                              mu_formula = x[1],
                              sigma_formula = x[2],
                              cor_formula = x_3,
                              copula = "vine",
                              usebam = FALSE,
                              n_cores = 24,
                              family_set = c("gauss"),
                              family = "nb")
  end <- Sys.time()
  print(end - start)
  return(res)
}
)

saveRDS(scDesign3Vine_result, "~/scDesign3/data/paper_writting/MARROW/scDesign3Vine_result.rds")

