library(scDesign3)
library(scater)
library(scran)


VISIUM_sce <- readRDS("~/scDesign3/data/sce_hvg/VISIUM_sce.rds")
logcounts(VISIUM_sce) <- log1p(counts(VISIUM_sce))

ncell <- dim(VISIUM_sce)[2]

ntrain <- round(ncell/2)

set.seed(123)
train_index <- sample(seq_len(ncell), ntrain, replace = FALSE)

train_sce <- VISIUM_sce[, train_index]
test_sce <- VISIUM_sce[, -train_index]
saveRDS(train_sce, "~/scDesign3/data/paper_writting/VISIUM/train_sce.rds")
saveRDS(test_sce, "~/scDesign3/data/paper_writting/VISIUM/test_sce.rds")

### Construct Formula Matrix
const_mu_formula <- "1"
discrete_mu_formula <- "cell_type"
continuous_mu_formula <- "s(spatial1, spatial2, k = 225, bs = 'gp')"

mu_formula <- as.factor(c(const_mu_formula, discrete_mu_formula, continuous_mu_formula))

const_sigma_formula <- "1"
discrete_sigma_formula <- "cell_type"
continuous_sigma_formula <- "s(pseudotime, k = 15, bs = 'gp')"

sigma_formula <- as.factor(c(const_sigma_formula, discrete_sigma_formula, continuous_sigma_formula))

ind_cor_formula <- "ind"
const_cor_formula <- "1"
discrete_cor_formula <- "cell_type"
continuous_cor_formula <- "spatial"

corr_formula <- as.factor(c(ind_cor_formula, const_cor_formula, discrete_cor_formula, continuous_cor_formula))


formula_mat <- expand.grid(mu_formua = mu_formula, sigma_formula = sigma_formula, corr_formula = corr_formula)
use_index <- c(1, 10, 2, 11, 20, 5, 14, 23, 3, 12, 9, 18)
formula_mat_VISIUM <- formula_mat[use_index, ]

saveRDS(formula_mat_VISIUM, "~/scDesign3/data/paper_writting/VISIUM/formula_mat.rds")

scDesign3Vine_result <- apply(formula_mat_VISIUM, 1, function(x) {
  print(x)
  start <- Sys.time()
  res <- scDesign3::scdesign3(sce = train_sce,
                              assay_use = "counts",
                              pseudotime = NULL,
                              celltype = "cell_type",
                              spatial = c("spatial1", "spatial2"),
                              other_covariates = NULL,
                              mu_formula = x[1],
                              sigma_formula = x[2],
                              cor_formula = x[3],
                              copula = "vine",
                              usebam = FALSE,
                              n_cores = 24,
                              family_set = c("gauss"),
                              family = "nb")
  end <- Sys.time()
  print(end - start)
  return(res)
}

)
saveRDS(scDesign3Vine_result, "~/scDesign3/data/paper_writting/VISIUM/scDesign3Vine_result.rds")
