library(scDesign3)
library(SingleCellExperiment)
library(Seurat)
library(lsa)
set.seed(123)

count_mat = read.table("~/SpatialBenchmarking/SpatialLocationData/Dataset10/Spatial_count.txt", header = TRUE)
count_mat = t(count_mat)
colnames(count_mat) = paste0("cell",1:dim(count_mat)[2])
true_location = read.table("~/SpatialBenchmarking/SpatialLocationData/Dataset10/Locations.txt", header = TRUE)

##select hvg
ngene <- 100
features = FindSpatiallyVariableFeatures(count_mat, spatial.location = true_location, selection.method = "moransi",nfeatures = ngene)
top.features = features[order(features$p.value),]
top.features= rownames(top.features[1:ngene,])

sce = SingleCellExperiment(list(counts = count_mat), colData = as.data.frame(true_location))
sce = sce[top.features,]

sce_list <- lapply(0:10, function(x) {
  set.seed(123)
  perturb_prop <- x/10
  n_cell <- round(dim(sce)[2]*perturb_prop)
  cell_index <- sample(1:dim(sce)[2], n_cell)
  new_spatial <- colData(sce)
  sampled_spatial <- new_spatial[cell_index,]
  shuffle_idx <- sample(1:n_cell, n_cell)
  shuffle_spatial <- sampled_spatial[shuffle_idx,]
  new_spatial[cell_index,] <- shuffle_spatial
  curr_sce <- sce
  colData(curr_sce) <- new_spatial
  curr_sce
})

names(sce_list) <- paste0(seq(0,100,10),"%")

scDesign3_res = lapply(sce_list, function(x){
  print(Sys.time())
  simu <- scdesign3(
    sce = x,
    assay_use = "counts",
    celltype = NULL,
    pseudotime = NULL,
    spatial = c("X", "Y"),
    other_covariates = NULL,
    mu_formula = "s(X, Y, bs = 'gp', k= 100)",
    sigma_formula = "1",
    family_use = "nb",
    n_cores = 50,
    usebam = FALSE,
    corr_formula = "1",
    copula = "vine",
    parallelization = "pbmcmapply"
  )
  return(simu)
})
names(scDesign3_res) <- names(sce_list)
saveRDS(scDesign3_res, "~/SpatialBenchmarking/SpatialLocationData/Dataset10/scDesign3_perturb_res.rds")


scDesign3_sce = lapply(1:length(sce_list), function(x){
  new_sce = SingleCellExperiment(list(counts = scDesign3_res[[x]]$new_count), colData = colData(sce_list[[x]]))
  return(new_sce)
})
names(scDesign3_sce) <- names(sce_list)
saveRDS(scDesign3_sce, "~/SpatialBenchmarking/SpatialLocationData/Dataset10/scDesign3_perturb_sce.rds")

truth = true_location
scDesign3_perturb_sce <- readRDS("~/SpatialBenchmarking/SpatialLocationData/Dataset10/scDesign3_perturb_sce.rds")
res_metric = matrix(0, ncol = length(scDesign3_perturb_sce), nrow = dim(true_location)[1])
for (i in 1:length(scDesign3_perturb_sce)) {
  curr_pseudo = as.matrix(colData(scDesign3_perturb_sce[[i]]))
  truth = as.matrix(true_location)
  res_metric[,i] = sapply(1:dim(true_location)[1], function(y){
    cosine(curr_pseudo[y,], truth[y,])
  })
}
rownames(res_metric)= rownames(truth)
colnames(res_metric)=  paste0(seq(0,100,10),"%")
saveRDS(res_metric,"~/SpatialBenchmarking/SpatialLocationData/Dataset10/cosine_similarity_perturb.rds")


