library(scDesign3)
library(Seurat)
library(SeuratObject)
library(SingleCellExperiment)
library(lsa)
set.seed(123)
data_txt  = "Seurat"
data_csv = c("Tangram", "novoSpaRc")
dataset = c(data_txt, data_csv)
dir = "~/SpatialBenchmarking/SpatialLocationData/Dataset10/"
count_mat = read.table(paste0(dir,"Spatial_count.txt"), header = TRUE)
count_mat = t(count_mat)
colnames(count_mat) = paste0("cell",1:dim(count_mat)[2])
true_location = read.table("~/SpatialBenchmarking/SpatialLocationData/Dataset10/Locations.txt", header = TRUE)

##select hvg
ngene <- 100
features = FindSpatiallyVariableFeatures(count_mat, spatial.location = true_location, selection.method = "moransi",nfeatures = ngene)
top.features = features[order(features$p.value),]
top.features= rownames(top.features[1:ngene,])

sce_list = lapply(dataset, function(x){
  # if(x%in% data_csv){
  #   pseudo_loc_file = paste0(dir,x,"_pseudo_location.csv")
  #   pseudo_loc = read.csv(pseudo_loc_file,row.names = 1)
  # }else{
  #   pseudo_loc = read.table(paste0(dir,x,"_pseudo_location.txt"), header = TRUE)
  # }
  pseudo_loc_file = paste0(dir,x,"_pseudo_location.rds")
  pseudo_loc = readRDS(paste0(dir,x,"_pseudo_location.rds"))
  sce = SingleCellExperiment(list(counts = count_mat), colData = as.data.frame(pseudo_loc))
  sce = sce[top.features,]
  return(sce)
})
names(sce_list) = dataset
## calculate cosine similarity between truth and  pseudo-location
res_metric = matrix(0, ncol = length(sce_list), nrow = dim(true_location)[1])
for (i in 1:length(sce_list)) {
  curr_pseudo = as.matrix(colData(sce_list[[i]]))
  truth = as.matrix(true_location)
  res_metric[,i] = sapply(1:dim(true_location)[1], function(y){
    cosine(curr_pseudo[y,], truth[y,])
  })
}
colnames(res_metric) = names(sce_list)
rownames(res_metric) = rownames(true_location)
saveRDS(res_metric, "~/SpatialBenchmarking/SpatialLocationData/Dataset10/cosine_similarity.rds")
sce_true = SingleCellExperiment(list(counts = count_mat), colData = as.data.frame(true_location))
sce_true = sce_true[top.features,]
sce_list = append(sce_list,sce_true)
names(sce_list)[4] = "truth"

scDesign3_res = lapply(sce_list, function(x){
  print(Sys.time())
  simu <- scdesign3(
    sce = x,
    assay_use = "counts",
    celltype = NULL,
    pseudotime = NULL,
    spatial = c("X", "Y"),
    other_covariates = NULL,
    mu_formula = "s(X, Y, bs = 'gp', k= 100)",
    sigma_formula = "1",
    family_use = "nb",
    n_cores = 50,
    usebam = FALSE,
    corr_formula = "1",
    copula = "vine",
    parallelization = "pbmcmapply"
  )
  return(simu)
})
names(scDesign3_res) <- names(sce_list)
saveRDS(scDesign3_res, "~/SpatialBenchmarking/SpatialLocationData/Dataset10/scDesign3_res.rds")

scDesign3_sce = lapply(1:(length(sce_list)-1), function(x){
  new_sce = SingleCellExperiment(list(counts = scDesign3_res[[x]]$new_count), colData = colData(sce_list[[x]]))
  return(new_sce)
})
names(scDesign3_sce) <- dataset
saveRDS(scDesign3_sce, "~/SpatialBenchmarking/SpatialLocationData/Dataset10/scDesign3_sce.rds")
