library(SingleCellExperiment)
library(useful)
library(irlba)
library(tidyverse)
library(scales)
library(umap)
library(lisi)
library(mgcv)
library(MASS)
library(pheatmap)
library(ComplexHeatmap)
library(mvtnorm)
library(Rfast)
library(mvnfast)
library(Matrix)
library(spcov)
library(umap)
library(circlize)
library(scater)
library(scran)
library(slingshot)
library(monocle3)
library(TSCAN)
library(scDesign3)
library(mclust)

get_earliest_principal_node <- function(cds, time_bin){
  cell_ids <- which(colData(cds)[, "grouping"] == time_bin)
  
  closest_vertex <-
    cds@principal_graph_aux[["UMAP"]]$pr_graph_cell_proj_closest_vertex
  closest_vertex <- as.matrix(closest_vertex[colnames(cds), ])
  root_pr_nodes <-
    igraph::V(principal_graph(cds)[["UMAP"]])$name[as.numeric(names
                                                              (which.max(table(closest_vertex[cell_ids,]))))]
  
  root_pr_nodes
}

# 
data_set <- c("EMBYRO", "PANCREAS", "GYRUS","dyngen_linear_simple", "dyngen_linear_tf10", "dyngen_linear_simple_tf10","dyngen_linear_tf30","dyngen_linear")
#data_set <- c("EMBYRO")


for(data in data_set){
  path <- paste0("/home/qingyang/scDesign3/data/Perturb/", data, "_500_train_sce.rds")
  train_sce <- readRDS(path)
  test_sce <- readRDS(paste0("/home/qingyang/scDesign3/data/Perturb/", data,"_500_test_sce.rds"))
  #sce <- cbind(train_sce, test_sce)
  sce <- train_sce
  
  train_sce <- train_sce
  sce <- sce
  
  train_idx <- which(colnames(sce) %in% colnames(train_sce))
  # perturb
  # 10
  train_sce_list <- lapply(0:10, function(x) {
    set.seed(234)
    perturb_prop <- x/10
    n_cell <- round(dim(train_sce)[2]*perturb_prop)
    cell_index <- sample(1:dim(train_sce)[2], n_cell)
    
    new_pseudotime <- colData(train_sce)$pseudotime
    new_pseudotime[cell_index] <- runif(n_cell)
    
    curr_sce <- train_sce
    colData(curr_sce)$pseudotime <- new_pseudotime
    curr_sce
  })
  names(train_sce_list) <- paste0(seq(0,100, by =10), "%")
  #slingshot
  FQnorm <- function(counts){
    rk <- apply(counts,2,rank,ties.method='min')
    counts.sort <- apply(counts,2,sort)
    refdist <- apply(counts.sort,1,median)
    norm <- apply(rk,2,function(r){ refdist[r] })
    rownames(norm) <- rownames(counts)
    return(norm)
  }
  
  assays(sce)$norm <- FQnorm(assays(sce)$counts)
  pca <- prcomp(t(log1p(assays(sce)$norm)), scale. = FALSE)
  rd1 <- pca$x[,1:2]
  rd2 <- uwot::umap(t(log1p(assays(sce)$norm)))
  colnames(rd2) <- c('UMAP1', 'UMAP2')
  reducedDims(sce) <- SimpleList(PCA = rd1, UMAP = rd2)
  if(!data %in% c("EMBYRO", "PANCREAS", "GYRUS")){
    cl1 <- Mclust(rd1)$classification
    colData(sce)$cell_type <- cl1
  }
  sce <- slingshot(sce, clusterLabels = 'cell_type', reducedDim = 'PCA')
  slingshot_pseudo = colData(sce)$slingPseudotime_1
  
  #monocle3
  cds <- new_cell_data_set(counts(sce), cell_metadata = colData(sce), gene_metadata = rowData(sce))
  rowdata <- data.frame(gene_short_name = rownames(sce))
  fData(cds) <- rowdata
  cds <- preprocess_cds(cds, num_dim = 50)
  cds <- reduce_dimension(cds, reduction_method = "UMAP")
  cds <- cluster_cells(cds)
  cds <- learn_graph(cds)
  #cds <- order_cells(cds)
  colData(cds)$grouping <- if_else(colData(cds)$pseudotime < 0.1, true =  1, false = 0)
  time_bin_start = 1
  
  cds <- order_cells(cds, root_pr_nodes = get_earliest_principal_node(cds, time_bin = time_bin_start))
  monocle3_pesudotime <- pseudotime(cds)
  
  ## TSCAN
  ## TSCAN
  #### Please check here, Qingyang
  procdata <- preprocess(counts(sce))
  if(dim(procdata)[1] < 10){
    procdata <- preprocess(counts(sce),minexpr_percent = 0.1)
  }
  
  lpsmclust <- exprmclust(procdata)
  #plotmclust(lpsmclust)
  lpsorder <- TSCANorder(lpsmclust,orderonly = T)
  TSCAN_pseudotime <- match(colnames(sce), lpsorder)
  TSCAN_pseudotime= (TSCAN_pseudotime -min(TSCAN_pseudotime))/(max(TSCAN_pseudotime) - min(TSCAN_pseudotime))
  
  new_psedu_list <- list("slingshot"= slingshot_pseudo, "monocle3"= monocle3_pesudotime ,"TSCAN"=TSCAN_pseudotime)
  names(new_psedu_list) <- c("slingshot", "monocle3", "TSCAN")
  train_sce_list2 <- lapply(new_psedu_list, function(x){
    curr_sce <- train_sce
    colData(curr_sce)$pseudotime <- x[train_idx]
    curr_sce
  })
  
  train_sce_list <- append(train_sce_list, train_sce_list2)
  
  train_sce_list <- lapply(train_sce_list, function(x) {
    logcounts(x) <- log1p(counts(x))
    x
  })
  
  scDesign3_result <- lapply(train_sce_list, function(x) {
    print(Sys.time())
    res <-  scdesign3(
      sce = x,
      assay_use = "logcounts",
      celltype = NULL,
      pseudotime = "pseudotime",
      spatial = NULL,
      other_covariates = NULL,
      mu_formula = "s(pseudotime, bs = 'cr', k = 10)",
      sigma_formula = 1,
      corr_formula = "ind",
      copula = "vine",
      n_cores = 50,
      family_use = "gaussian"
    )
    
    return(res)
  })
  
  #saveRDS(scDesign3_result, paste0("~/scDesign3/data/pseudotime_quality/original/", data, "_scDesign3.rds"))
  truth <- colData(train_sce)$pseudotime
  res <- matrix(0, nrow = length(train_sce_list), ncol = 2)
  colnames(res) <- c("BIC", "R2")
  rownames(res) <- names(train_sce_list)
  for (i in 1:length(train_sce_list)) {
    x <- scDesign3_result[[i]]
    new_sce <- SingleCellExperiment(list(counts = x$new_count), colData = colData(train_sce_list[[i]]))
    res[i,"BIC"] <- x$model_bic[1]
    res[i,"R2"] <- (cor(truth, colData(new_sce)$pseudotime))^2
    path <- paste0("~/scDesign3/data/pseudotime_quality/original/")
    curr_name <- paste0(data,names(train_sce_list)[i],"_sce.rds")
    full_path<-paste0(path, curr_name)
    #saveRDS(new_sce, full_path)
  }
  saveRDS(res, paste0("~/scDesign3/data/pseudotime_quality/gaussian/", data, "_metric.rds"))
}