library(scDesign3)
library(scater)
library(scran)


PANCREAS_sce <- readRDS("~/scDesign3/data/sce_hvg/PANCREAS_sce.rds")
logcounts(PANCREAS_sce) <- log1p(counts(PANCREAS_sce))

ncell <- dim(PANCREAS_sce)[2]

ntrain <- round(ncell/2)

set.seed(123)
train_index <- sample(seq_len(ncell), ntrain, replace = FALSE)

train_sce <- PANCREAS_sce[, train_index]
test_sce <- PANCREAS_sce[, -train_index]
saveRDS(train_sce, "~/scDesign3/data/paper_writing/PANCREAS/train_sce.rds")
saveRDS(test_sce, "~/scDesign3/data/paper_writing/PANCREAS/test_sce.rds")

### Construct Formula Matrix
const_mu_formula <- "1"
discrete_mu_formula <- "cell_type"
continuous_mu_formula <- "s(pseudotime, k = 10, bs = 'cr')"

mu_formula <- as.factor(c(const_mu_formula, discrete_mu_formula, continuous_mu_formula))

const_sigma_formula <- "1"
discrete_sigma_formula <- "cell_type"
continuous_sigma_formula <- "s(pseudotime, k = 3, bs = 'cr')"

sigma_formula <- as.factor(c(const_sigma_formula, discrete_sigma_formula, continuous_sigma_formula))

ind_cor_formula <- "ind"
const_cor_formula <- "1"
discrete_cor_formula <- "cell_type"
continuous_cor_formula <- "pseudotime"

corr_formula <- as.factor(c(ind_cor_formula, const_cor_formula, discrete_cor_formula, continuous_cor_formula))


formula_mat <- expand.grid(mu_formua = mu_formula, sigma_formula = sigma_formula, corr_formula = corr_formula)
use_index <- c(1, 10, 2, 11, 20, 5, 14, 23, 3, 12, 30, 9, 18, 36)
formula_mat_PANCREAS <- formula_mat[use_index, ]

saveRDS(formula_mat_PANCREAS, "~/scDesign3/data/paper_writing/PANCREAS/formula_mat.rds")
# mu_formula_vec <- c(rep(discrete_mu_formula, 6), rep(continuous_mu_formula, 6))
# sigma_formula_vec <- c(rep(c(const_sigma_formula, discrete_sigma_formula), 3),
#                        rep(c(const_sigma_formula, continuous_sigma_formula), 3))
# cor_formula_vec <- c(rep(c(ind_cor_formula, const_cor_formula, hetero_cor_formula1), 2),
#                      rep(c(ind_cor_formula, const_cor_formula, hetero_cor_formula2), 2))
# 
# cov_use <- c(rep("cell_type", 6), rep("pseudotime", 6))
# 
# formula_mat <- cbind(mu_formula_vec, sigma_formula_vec, cor_formula_vec, cov_use)

scDesign3Gaussian_result <- apply(formula_mat_PANCREAS, 1, function(x) {
  print(x)
  start <- Sys.time()
  res <- scDesign3::scdesign3(sce = train_sce,
                              assay_use = "counts",
                              pseudotime = "pseudotime",
                              celltype = "cell_type",
                              spatial = NULL,
                              other_covariates = NULL,
                              mu_formula = x[1],
                              sigma_formula = x[2],
                              cor_formula = x[3],
                              copula = "gaussian",
                              usebam = FALSE,
                              n_cores = 24,
                              family = "nb"
  )
  end <- Sys.time()
  print(end - start)
  return(res)
})

saveRDS(scDesign3Gaussian_result, "~/scDesign3/data/paper_writing/PANCREAS/scDesign3Gaussian_result.rds")

scDesign3Vine_result <- apply(formula_mat_PANCREAS, 1, function(x) {
  print(x)
  start <- Sys.time()
  res <- scDesign3::scdesign3(sce = train_sce,
                              assay_use = "counts",
                              pseudotime = "pseudotime",
                              celltype = "cell_type",
                              spatial = NULL,
                              other_covariates = NULL,
                              mu_formula = x[1],
                              sigma_formula = x[2],
                              cor_formula = x[3],
                              copula = "vine",
                              usebam = FALSE,
                              n_cores = 24,
                              family_set = c("gauss"),
                              family = "nb")
  end <- Sys.time()
  print(end - start)
  return(res)
}

)
saveRDS(scDesign3Vine_result, "~/scDesign3/data/paper_writing/PANCREAS/scDesign3Vine_result.rds")
