library(scDesign3)
library(scater)
library(scran)
library(tidyverse)

dataname <- "CITE"
train_sce <- readRDS(paste0("/home/dongyuan/scDesign3/data/sce_hvg_train/", dataname, "_sce.rds"))

### Construct Formula Matrix
#continuous_mu_formula <- "s(spatial1, spatial2, bs = 'tp', k=400)"
### Construct Formula Matrix
const_mu_formula <- "1"
discrete_mu_formula <- "cell_type"
continuous_mu_formula <- "none"

mu_formula <- as.factor(c(const_mu_formula, discrete_mu_formula, continuous_mu_formula))

const_sigma_formula <- "1"
discrete_sigma_formula <- "cell_type"
continuous_sigma_formula <- "none"

sigma_formula <- as.factor(c(const_sigma_formula, discrete_sigma_formula, continuous_sigma_formula))

ind_cor_formula <- "ind"
const_cor_formula <- "1"
discrete_cor_formula <- "cell_type"
continuous_cor_formula <- "none"

corr_formula <- as.factor(c(ind_cor_formula, const_cor_formula, discrete_cor_formula, continuous_cor_formula))

formula_mat <- expand.grid(mu_formua = mu_formula, sigma_formula = sigma_formula, corr_formula = corr_formula)
formula_mat <- tibble(formula_mat) %>% dplyr::filter(sigma_formula == "1" | sigma_formula == mu_formula) %>% 
  dplyr::filter(corr_formula != "cell_type + batch") %>% 
  dplyr::filter(mu_formua != "none" & sigma_formula != "none" & corr_formula != "none")
print(formula_mat) 
saveRDS(formula_mat, paste0("/home/dongyuan/scDesign3/data/sce_hvg_scDesign3/", dataname, "_formula_mat.rds"))

### Start

### Gaussian
scDesign3Gaussian_result <- apply(formula_mat, 1, function(x) {
  x <- as.matrix(x)
  cat(x)
  cat("\n")
  start <- Sys.time()
  res <- scDesign3::scdesign3(sce = train_sce,
                              assay_use = "counts",
                              celltype = "cell_type",
                              pseudotime = NULL,
                              spatial = NULL,
                              other_covariates = NULL,
                              mu_formula = x[1],
                              sigma_formula = x[2],
                              family_use = "nb",
                              n_cores = 50,
                              usebam = FALSE,
                              corr_formula = x[3],
                              copula = "gaussian",
                              DT = TRUE,
                              pseudo_obs = FALSE,
                              family_set = c("gauss", "indep"),
                              nonnegative = TRUE,
                              nonzerovar = TRUE,
                              return_model = FALSE)
  
  end <- Sys.time()
  print(end - start)
  return(res)
})

saveRDS(scDesign3Gaussian_result, paste0("/home/dongyuan/scDesign3/data/sce_hvg_scDesign3/", dataname, "_scDesign3Gaussian_result.rds"))

## Vine
scDesign3Vine_result <- apply(formula_mat, 1, function(x) {
  x <- as.matrix(x)
  cat(x)
  cat("\n")
  start <- Sys.time()
  res <- scDesign3::scdesign3(sce = train_sce, 
                              assay_use = "counts", 
                              celltype = "cell_type", 
                              pseudotime = NULL, 
                              spatial = NULL, 
                              other_covariates = NULL, 
                              mu_formula = x[1], 
                              sigma_formula = x[2], 
                              family_use = "nb", 
                              n_cores = 1, 
                              usebam = FALSE, 
                              corr_formula = x[3], 
                              copula = "vine", 
                              DT = TRUE, 
                              pseudo_obs = FALSE, 
                              family_set = c("gauss", "indep"), 
                              nonnegative = TRUE, 
                              nonzerovar = TRUE, 
                              return_model = FALSE)
  end <- Sys.time()
  print(end - start)
  return(res)
})

saveRDS(scDesign3Vine_result, paste0("/home/dongyuan/scDesign3/data/sce_hvg_scDesign3/", dataname, "_scDesign3Vine_result.rds"))