library(scDesign3)
library(scater)
library(scran)
library(tidyverse)

dataname <- "MARROW"
train_sce <- readRDS(paste0("/home/dongyuan/scDesign3/data/sce_hvg_train/", dataname, "_sce.rds"))

coldata <- colData(train_sce)
coldata$pseudotime1[coldata$pseudotime1 < 0] <- 0
coldata$pseudotime2[coldata$pseudotime2 < 0] <- 0
colData(train_sce) <- coldata

const_mu_formula <- "1"
discrete_mu_formula <- "cell_type"
continuous_mu_formula <- "s(pseudotime1, k = 10, bs = 'cr') + s(pseudotime2, k = 10, bs = 'cr')"

mu_formula <- as.factor(c(const_mu_formula, discrete_mu_formula, continuous_mu_formula))

const_sigma_formula <- "1"
discrete_sigma_formula <- "cell_type"
continuous_sigma_formula <- "s(pseudotime1, k = 3, bs = 'cr') + s(pseudotime2, k = 3, bs = 'cr')"

sigma_formula <- as.factor(c(const_sigma_formula, discrete_sigma_formula, continuous_sigma_formula))

ind_cor_formula <- "ind"
const_cor_formula <- "1"
discrete_cor_formula <- "cell_type"
continuous_cor_formula <- "pseudotime"

corr_formula <- as.factor(c(ind_cor_formula, const_cor_formula, discrete_cor_formula, continuous_cor_formula))

formula_mat <- expand.grid(mu_formua = mu_formula, sigma_formula = sigma_formula, corr_formula = corr_formula)
formula_mat <- data.frame(formula_mat)
formula_mat <- tibble(formula_mat) %>% .[c(1, 2, 3, 5, 9, 10, 11, 12, 14, 18, 20, 23, 30, 36), ]
print(formula_mat)
saveRDS(formula_mat, paste0("/home/dongyuan/scDesign3/data/sce_hvg_scDesign3/", dataname, "_formula_mat.rds"))

### Start

### Gaussian

scDesign3Gaussian_result <- apply(formula_mat, 1, function(x) {
  x <- as.matrix(x)
  cat(x)
  cat("\n")
  start <- Sys.time()
  if(x[3] == "pseudotime") {
    corr_formula <- c("pseudotime1", "pseudotime2")
  } else {
    corr_formula <- x[3]
  }
  res <- scDesign3::scdesign3(sce = train_sce,
                              assay_use = "counts",
                              celltype = "cell_type",
                              pseudotime = c("pseudotime1", "pseudotime2"),
                              spatial = NULL,
                              other_covariates = NULL,
                              mu_formula = x[1],
                              sigma_formula = x[2],
                              family_use = "nb",
                              n_cores = 50,
                              usebam = FALSE,
                              corr_formula = corr_formula,
                              copula = "gaussian",
                              DT = TRUE,
                              pseudo_obs = FALSE,
                              family_set = c("gauss", "indep"),
                              nonnegative = TRUE,
                              nonzerovar = TRUE,
                              return_model = FALSE)
  
  end <- Sys.time()
  print(end - start)
  return(res)
})

saveRDS(scDesign3Gaussian_result, paste0("/home/dongyuan/scDesign3/data/sce_hvg_scDesign3/", dataname, "_scDesign3Gaussian_result.rds"))

# Vine
scDesign3Vine_result <- apply(formula_mat, 1, function(x) {
  x <- as.matrix(x)
  cat(x)
  cat("\n")
  start <- Sys.time()
  if(x[3] == "pseudotime") {
    corr_formula <- c("pseudotime1", "pseudotime2")
  } else {
    corr_formula <- x[3]
  }
  res <- scDesign3::scdesign3(sce = train_sce,
                              assay_use = "counts",
                              celltype = "cell_type",
                              pseudotime = c("pseudotime1", "pseudotime2"),
                              spatial = NULL,
                              other_covariates = NULL,
                              mu_formula = x[1],
                              sigma_formula = x[2],
                              family_use = "nb",
                              n_cores = 50,
                              usebam = FALSE,
                              corr_formula = corr_formula,
                              copula = "vine",
                              DT = TRUE,
                              pseudo_obs = FALSE,
                              family_set = c("gauss", "indep"),
                              nonnegative = TRUE,
                              nonzerovar = TRUE,
                              return_model = FALSE)

  end <- Sys.time()
  print(end - start)
  return(res)
})

saveRDS(scDesign3Vine_result, paste0("/home/dongyuan/scDesign3/data/sce_hvg_scDesign3/", dataname, "_scDesign3Vine_result.rds"))