library(scDesign3)
library(scater)
library(scran)
library(tidyverse)
library(SingleCellExperiment)

cell_metadata_bonemarrow <- read_delim("/home/dongyuan/scDesign3/data/scReadSim/cell_metadata_bonemarrow.csv", 
                                       delim = "\t", escape_double = FALSE, 
                                       trim_ws = TRUE)
count_mat <- read.table("/home/dongyuan/scDesign3/data/scReadSim/BoneMarrow_62016_chr1.countmatrix.txt", row.names = 1) %>% as.matrix()
colnames(count_mat) <- cell_metadata_bonemarrow$cell

background_mat <- read.table("/home/dongyuan/scDesign3/data/scReadSim/BoneMarrow_62016_chr1.COMPLE.countmatrix.txt", row.names = 1) %>% as.matrix()
colnames(background_mat) <- cell_metadata_bonemarrow$cell

dim(count_mat)
dim(background_mat)

min(rowSums(count_mat))
min(rowSums(background_mat))

forecount_sce <- SingleCellExperiment::SingleCellExperiment(list(counts = count_mat))
backcount_sce <- SingleCellExperiment::SingleCellExperiment(list(counts = background_mat))

colData(forecount_sce) <- DataFrame(cell_metadata_bonemarrow)
colData(backcount_sce) <- DataFrame(cell_metadata_bonemarrow)
colData(forecount_sce)$cell_type <- cell_metadata_bonemarrow$cell_label
colData(backcount_sce)$cell_type <- cell_metadata_bonemarrow$cell_label

rare_celltype <- names(sort(table(cell_metadata_bonemarrow$cell_label)))[1:3]

colnames(forecount_sce) <- colnames(count_mat)
colnames(backcount_sce) <- colnames(background_mat)

forecount_sce <- forecount_sce[,!colData(forecount_sce)$cell_type %in% rare_celltype]
backcount_sce <- backcount_sce[,!colData(backcount_sce)$cell_type %in% rare_celltype]

saveRDS(forecount_sce, "/home/dongyuan/scDesign3/data/scReadSim/forecount_sce.rds")
saveRDS(backcount_sce, "/home/dongyuan/scDesign3/data/scReadSim/backcount_sce.rds")


## Simulate new data
### Construct Formula Matrix
const_mu_formula <- "1"
discrete_mu_formula <- "cell_type"
continuous_mu_formula <- "s(pseudotime, k = 10, bs = 'cr')"

mu_formula <- as.factor(c(const_mu_formula, discrete_mu_formula, continuous_mu_formula))

const_sigma_formula <- "1"
discrete_sigma_formula <- "cell_type"
continuous_sigma_formula <- "s(pseudotime, k = 3, bs = 'cr')"

sigma_formula <- as.factor(c(const_sigma_formula, discrete_sigma_formula, continuous_sigma_formula))

ind_cor_formula <- "ind"
const_cor_formula <- "1"
discrete_cor_formula <- "cell_type"
continuous_cor_formula <- "pseudotime"

corr_formula <- as.factor(c(ind_cor_formula, const_cor_formula, discrete_cor_formula, continuous_cor_formula))


formula_mat <- expand.grid(mu_formua = mu_formula, sigma_formula = sigma_formula, corr_formula = corr_formula)
use_index <- c(11, 14, 20, 23)
formula_mat <- formula_mat[use_index, ]
print(formula_mat)
saveRDS(formula_mat, paste0("/home/dongyuan/scDesign3/data/scReadSim/formula_mat.rds"))



scDesign3Gaussian_result <- apply(formula_mat, 1, function(x) {
  x <- as.matrix(x)
  cat(x)
  cat("\n")
  start <- Sys.time()
  res <- scDesign3::scdesign3(sce = forecount_sce,
                              assay_use = "counts",
                              celltype = "cell_type",
                              pseudotime = NULL,
                              spatial = NULL,
                              other_covariates = NULL,
                              mu_formula = x[1],
                              sigma_formula = x[2],
                              family_use = "zip",
                              n_cores = 50,
                              usebam = FALSE,
                              corr_formula = x[3],
                              copula = "gaussian",
                              DT = TRUE,
                              pseudo_obs = FALSE,
                              family_set = c("gauss", "indep"),
                              nonnegative = TRUE,
                              nonzerovar = FALSE,
                              return_model = FALSE)

  end <- Sys.time()
  print(end - start)
  return(res)
})

saveRDS(scDesign3Gaussian_result, paste0("/home/dongyuan/scDesign3/data/scReadSim/forecount_scDesign3Gaussian_result.rds"))

scDesign3Gaussian_result_nb <- apply(formula_mat, 1, function(x) {
  x <- as.matrix(x)
  cat(x)
  cat("\n")
  start <- Sys.time()
  res <- scDesign3::scdesign3(sce = forecount_sce,
                              assay_use = "counts",
                              celltype = "cell_type",
                              pseudotime = NULL,
                              spatial = NULL,
                              other_covariates = NULL,
                              mu_formula = x[1],
                              sigma_formula = x[2],
                              family_use = "nb",
                              n_cores = 50,
                              usebam = FALSE,
                              corr_formula = x[3],
                              copula = "gaussian",
                              DT = TRUE,
                              pseudo_obs = FALSE,
                              family_set = c("gauss", "indep"),
                              nonnegative = TRUE,
                              nonzerovar = FALSE,
                              return_model = FALSE)
  
  end <- Sys.time()
  print(end - start)
  return(res)
})

saveRDS(scDesign3Gaussian_result_nb, paste0("/home/dongyuan/scDesign3/data/scReadSim/forecount_scDesign3Gaussian_result_nb.rds"))

scDesign3Gaussian_result <- apply(formula_mat, 1, function(x) {
  x <- as.matrix(x)
  cat(x)
  cat("\n")
  start <- Sys.time()
  res <- scDesign3::scdesign3(sce = backcount_sce,
                              assay_use = "counts",
                              celltype = "cell_type",
                              pseudotime = NULL,
                              spatial = NULL,
                              other_covariates = NULL,
                              mu_formula = x[1],
                              sigma_formula = x[2],
                              family_use = "zip",
                              n_cores = 50,
                              usebam = FALSE,
                              corr_formula = x[3],
                              copula = "gaussian",
                              DT = TRUE,
                              pseudo_obs = FALSE,
                              family_set = c("gauss", "indep"),
                              nonnegative = TRUE,
                              nonzerovar = FALSE,
                              return_model = FALSE)
  
  end <- Sys.time()
  print(end - start)
  return(res)
})

saveRDS(scDesign3Gaussian_result, paste0("/home/dongyuan/scDesign3/data/scReadSim/backcount_scDesign3Gaussian_result.rds"))

scDesign3Gaussian_result_nb <- apply(formula_mat, 1, function(x) {
  x <- as.matrix(x)
  cat(x)
  cat("\n")
  start <- Sys.time()
  res <- scDesign3::scdesign3(sce = backcount_sce,
                              assay_use = "counts",
                              celltype = "cell_type",
                              pseudotime = NULL,
                              spatial = NULL,
                              other_covariates = NULL,
                              mu_formula = x[1],
                              sigma_formula = x[2],
                              family_use = "nb",
                              n_cores = 50,
                              usebam = FALSE,
                              corr_formula = x[3],
                              copula = "gaussian",
                              DT = TRUE,
                              pseudo_obs = FALSE,
                              family_set = c("gauss", "indep"),
                              nonnegative = TRUE,
                              nonzerovar = FALSE,
                              return_model = FALSE)
  
  end <- Sys.time()
  print(end - start)
  return(res)
})

saveRDS(scDesign3Gaussian_result_nb, paste0("/home/dongyuan/scDesign3/data/scReadSim/backcount_scDesign3Gaussian_result_nb.rds"))

