library(scDesign3)

MOBSC_sce <- readRDS(file = "/home/dongyuan/scDesign3/data/sce_hvg/MOBSC_sce.rds")

MOBSC_data <- construct_data(
  sce = MOBSC_sce,
  assay_use = "counts",
  celltype = "cell_type",
  pseudotime = NULL,
  spatial = NULL,
  other_covariates = NULL,
  corr_by = "1"
)

MOBSC_marginal <- fit_marginal(
  data = MOBSC_data,
  predictor = "gene",
  mu_formula = "cell_type",
  sigma_formula = "cell_type",
  family_use = "nb",
  n_cores = 50,
  usebam = FALSE, 
  parallelization = "pbmcmapply"
  
)

MOBSC_copula <- fit_copula(
  sce = MOBSC_sce,
  assay_use = "counts",
  marginal_list = MOBSC_marginal,
  family_use = "nb",
  copula = "gaussian",
  n_cores = 10,
  new_covariate = NULL,
  input_data = MOBSC_data$dat
)

MOBSC_para <- extract_para(
  sce = MOBSC_sce,
  marginal_list = MOBSC_marginal,
  n_cores = 10,
  family_use = "nb",
  new_covariate = NULL
)

MOBSC_newcount <- simu_new(
  sce = MOBSC_sce,
  mean_mat = MOBSC_para$mean_mat,
  sigma_mat = MOBSC_para$sigma_mat,
  zero_mat = MOBSC_para$zero_mat,
  quantile_mat = NULL,
  copula_list = MOBSC_copula$copula_list,
  n_cores = 10,
  family_use = "nb",
  input_data = MOBSC_data$dat,
  new_covariate = MOBSC_data$new_covariate
)

saveRDS(MOBSC_para, "/home/dongyuan/scDesign3/data/deconvolution/MOBSC_para.rds")
saveRDS(MOBSC_newcount, "/home/dongyuan/scDesign3/data/deconvolution/MOBSC_newcount.rds")




MOBSP_sce <- readRDS(file = "/home/dongyuan/scDesign3/data/sce_hvg/MOBSP_sce.rds")

MOBSP_data <- construct_data(
  sce = MOBSP_sce,
  assay_use = "counts",
  celltype = NULL,
  pseudotime = NULL,
  spatial = c("spatial1", "spatial2"),
  other_covariates = NULL,
  corr_by = "1"
)

MOBSP_marginal <- fit_marginal(
  data = MOBSP_data,
  predictor = "gene",
  mu_formula = "s(spatial1, spatial2, bs = 'gp', k = 200)",
  sigma_formula = "1",
  family_use = "nb",
  n_cores = 50,
  usebam = FALSE, 
  parallelization = "pbmcmapply"
  
)

MOBSP_copula <- fit_copula(
  sce = MOBSP_sce,
  assay_use = "counts",
  marginal_list = MOBSP_marginal,
  family_use = "nb",
  copula = "gaussian",
  n_cores = 10,
  new_covariate = NULL,
  input_data = MOBSP_data$dat
)

MOBSP_para <- extract_para(
  sce = MOBSP_sce,
  marginal_list = MOBSP_marginal,
  n_cores = 10,
  family_use = "nb",
  new_covariate = NULL
)

MOBSP_newcount <- simu_new(
  sce = MOBSP_sce,
  mean_mat = MOBSP_para$mean_mat,
  sigma_mat = MOBSP_para$sigma_mat,
  zero_mat = MOBSP_para$zero_mat,
  quantile_mat = NULL,
  copula_list = MOBSP_copula$copula_list,
  n_cores = 10,
  family_use = "nb",
  input_data = MOBSP_data$dat,
  new_covariate = MOBSP_data$new_covariate
)

saveRDS(MOBSP_para, "/home/dongyuan/scDesign3/data/deconvolution/MOBSP_para.rds")
saveRDS(MOBSP_newcount, "/home/dongyuan/scDesign3/data/deconvolution/MOBSP_newcount.rds")

