library(SingleCellExperiment)
library(tidyverse)
library(ggpubr)
library(cowplot)
library(ggrepel)
library(ggnewscale)
library(ggh4x)
library(viridis)
library(ggrastr)
library(caret)
library(pbmcapply)
theme_set(theme_bw())

set.seed(123)
fitControl <- trainControl(## 5-fold CV
  method = "repeatedcv",
  number = 5,
  ## repeated five times
  repeats = 5)

ml_method_list <- c("gbm", "ranger", "svmRadial") ## ML methods
spatial_data_name <- c("VISIUM", "SLIDE", "OVARIAN", "ACINAR") ## Four spatial datasets

train_sce_list <- lapply(spatial_data_name, function(x) {
  readRDS(paste0("~/scDesign3/data/sce_hvg_train/", x, "_sce.rds"))
})

test_sce_list <- lapply(spatial_data_name, function(x) {
  readRDS(paste0("~/scDesign3/data/sce_hvg_test/", x, "_sce.rds"))
})

scDesign3_count_list <- lapply(spatial_data_name, function(x) {
  readRDS(paste0("~/scDesign3/data/sce_hvg_scDesign3/", x, "_scDesign3Vine_result.rds"))[[7]]$new_count
})

lapply(seq_len(length(spatial_data_name)), function(i) {
    dat_rmse <- pbmcapply::pbmclapply(1:100, function(j) {
    gene_names <- rownames(train_sce_list[[i]])
    x1 <- data.frame(Expression = as.numeric(log1p(counts(train_sce_list[[i]])[j, ])),
                     spatial1 = colData(train_sce_list[[i]])$spatial1, 
                     spatial2 = colData(train_sce_list[[i]])$spatial2)
    
    x2 <- data.frame(Expression = as.numeric(log1p(scDesign3_count_list[[i]][j, ])),
                     spatial1 = colData(train_sce_list[[i]])$spatial1, 
                     spatial2 = colData(train_sce_list[[i]])$spatial2)
    x3 <- data.frame(Expression = as.numeric(log1p(counts(test_sce_list[[i]])[j, ])),
                     spatial1 = colData(test_sce_list[[i]])$spatial1, 
                     spatial2 = colData(test_sce_list[[i]])$spatial2)
    
    
    
    rmse_tab <- sapply(ml_method_list, function(z) {
      Fit1 <- train(Expression ~ ., data = x1, 
                    method = z, 
                    trControl = fitControl,
                    verbose = FALSE)
      Fit2 <- train(Expression ~ ., data = x2, 
                    method = z, 
                    trControl = fitControl,
                    verbose = FALSE)
      y1 <- predict(Fit1, newdata = x3)
      y2 <- predict(Fit2, newdata = x3)
      
      rmse1 <- sqrt(mean((x3$Expression - y1)^2))
      rmse2 <- sqrt(mean((x3$Expression - y2)^2))
      res_rmse <- c(rmse1, rmse2)
      names(res_rmse) <- c("Train", "scDesign3")
      return(res_rmse)
    })
    
    rmse_tbl <- rmse_tab %>% as_tibble(rownames = "Data") %>% dplyr::mutate(Gene = gene_names[j]) %>% tidyr::pivot_longer(cols = ml_method_list, names_to = "Method", values_to = "RMSE")
    
    return(rmse_tbl)
  }, mc.cores = 20)
    dat_rmse <- bind_rows(dat_rmse)
    print(paste0(spatial_data_name[i], " finished!"))
  saveRDS(object = dat_rmse, paste0("/home/dongyuan/scDesign3/data/spatial_prediction/", spatial_data_name[i], "_rmse.rds"))
})