apply_PCAHC <- function(sce, params, k) {
  tryCatch({
    dat <- logcounts(sce)
    st <- system.time({
      pca <- prcomp(t(dat), center = TRUE, scale. = FALSE)
      pca <- pca$x[, seq_len(params$nPC), drop = FALSE]
      hcl <- hclust(dist(pca), method = "ward.D2")
      cluster <- cutree(hcl, k = k)
      names(cluster) = colnames(dat)
    })
    
    st <- c(user.self = st[["user.self"]], sys.self = st[["sys.self"]], 
            user.child = st[["user.child"]], sys.child = st[["sys.child"]],
            elapsed = st[["elapsed"]])
    list(st = st, cluster = cluster, est_k = NA)
  }, error = function(e) {
    list(st = c(user.self = NA, sys.self = NA, user.child = NA, sys.child = NA,
                elapsed = NA), 
         cluster = structure(rep(NA, ncol(sce)), names = colnames(sce)), 
         est_k = NA)
  })
}

res.summary <- function(res) {res %>%
  dplyr::group_by(dataset, method, run, k, resolution) %>%
  dplyr::summarize(ARI = aricode::ARI(cluster, trueclass),
                   AMI = aricode::AMI(cluster, trueclass),
                   NMI = aricode::NMI(cluster, trueclass),
                   truenclust = length(unique(trueclass)),
                   estnclust = unique(est_k),
                   elapsed = stats::median(elapsed)) %>%
  tidyr::separate(dataset, sep = "_", into = c("sce", "filtering",
                                               "dataset")) %>%
  dplyr::select(-sce) %>% dplyr::ungroup()}
