require(Rfast)

createSummary <- function(train_sce, 
                          test_sce, 
                          scDesign3Vine_result, 
                          formula_mat,
                          ngene_cor = 100) {
  if(is.vector(formula_mat)) {
    model_name <- formula_mat
  } else {
    model_name <- apply(formula_mat, 1, function(x) {
      paste0(x, collapse = "_")
    })
  }

  ## Extract AIC
  if(is.null(scDesign3Vine_result[[1]]$model_aic)) {
    aic_mat <- t(sapply(scDesign3Vine_result, function(x) x$model.aic))
  } else aic_mat <- t(sapply(scDesign3Vine_result, function(x) x$model_aic))

  colnames(aic_mat) <- c("Marginal_AIC", "Copula_AIC", "Total_AIC")
  rownames(aic_mat) <- model_name
  ref_mat <- matrix(NA, 1, 3)
  colnames(ref_mat) <- c("Marginal_AIC", "Copula_AIC", "Total_AIC")
  rownames(ref_mat) <- c("Test")
  aic_mat <- rbind(ref_mat, aic_mat)
  
  ## Extract bic
  if(is.null(scDesign3Vine_result[[1]]$model_bic)) {
    bic_mat <- t(sapply(scDesign3Vine_result, function(x) x$model.bic))
  } else bic_mat <- t(sapply(scDesign3Vine_result, function(x) x$model_bic))
  colnames(bic_mat) <- c("Marginal_BIC", "Copula_BIC", "Total_BIC")
  rownames(bic_mat) <- model_name
  ref_mat <- matrix(NA, 1, 3)
  colnames(ref_mat) <- c("Marginal_BIC", "Copula_BIC", "Total_BIC")
  rownames(ref_mat) <- c("Test")
  bic_mat <- rbind(ref_mat, bic_mat)
  
  
  ## Construct PCA
  test_pca_fit <- irlba::prcomp_irlba(t(log1p(counts(test_sce))), 
                                      center = TRUE, 
                                      scale. = TRUE, 
                                      n = 50)
  test_pca_tbl <- predict(test_pca_fit, newdata= t(log1p(counts(test_sce)))) %>% as_tibble() %>% dplyr::mutate(Method = "test_data") %>% dplyr::bind_cols(as_tibble(colData(test_sce)))
  train_pca_tbl <- predict(test_pca_fit, newdata= t(log1p(counts(train_sce)))) %>% as_tibble() %>% dplyr::mutate(Method = "ref") %>% dplyr::bind_cols(as_tibble(colData(train_sce)))
  
  if(is.null(scDesign3Vine_result[[1]]$new_count)) {
    scDesign3_count_list <- lapply(scDesign3Vine_result, function(x) x$new.count)
  } else scDesign3_count_list <- lapply(scDesign3Vine_result, function(x) x$new_count)
  
  scDesign3_pca_list <- lapply(scDesign3_count_list, function(x, pca.fit) {mat <- predict(pca.fit, newdata = t(log1p(x)))
  
  as_tibble(mat) %>% dplyr::bind_cols(as_tibble(colData(train_sce)))
  }, pca.fit = test_pca_fit)
  names(scDesign3_pca_list) <- model_name
  scDesign3_pca_tbl <- bind_rows(scDesign3_pca_list, .id = "Method")
  
  scDesign3_pca_tbl <- scDesign3_pca_tbl %>% dplyr::mutate(Method = factor(Method, levels = model_name))
  
  pca_tbl <- bind_rows(test_pca_tbl, scDesign3_pca_tbl) %>% dplyr::mutate(Method = factor(Method, levels = c("test_data", model_name)))
  pca_tbl_list <- pca_tbl %>% group_by(Method) %>% group_split()
  # pca_lisi_label <- sapply(pca_tbl_list, function(x) {
  #   tbl <- dplyr::bind_rows(train_pca_tbl, x)
  #   
  #   tryCatch(expr = {
  #     lisi_fit <- lisi::compute_lisi(tbl[, c("PC1", "PC2")], meta_data = tbl[, "Method"], label_colnames = "Method")
  #     lisi_fit %>% unlist() %>% mean() %>% round(4)
  #   },
  #   error = function(error) {NA})
  # })

  ## UMAP
  test_umap_fit <- umap::umap((test_pca_tbl %>% dplyr::select(starts_with("PC"))))
  
  test_umap_tbl <- predict(object = test_umap_fit, data= (test_pca_tbl %>% dplyr::select(starts_with("PC")))) %>% as_tibble() %>% dplyr::rename(UMAP1 = "V1", UMAP2 = "V2") %>% dplyr::mutate(Method = "test_data") %>% dplyr::bind_cols(as_tibble(colData(test_sce)))
  train_umap_tbl <- predict(object = test_umap_fit, data= (train_pca_tbl %>% dplyr::select(starts_with("PC")))) %>% as_tibble() %>% dplyr::rename(UMAP1 = "V1", UMAP2 = "V2") %>% dplyr::mutate(Method = "ref") %>% dplyr::bind_cols(as_tibble(colData(train_sce)))
  
  
  scDesign3_umap_list <- lapply(scDesign3_pca_list, function(x, umap.fit) {mat <- predict(umap.fit, (x %>% dplyr::select(starts_with("PC"))))
  as_tibble(mat)  %>% dplyr::rename(UMAP1 = "V1", UMAP2 = "V2")%>% dplyr::bind_cols(as_tibble(colData(train_sce)))
  }, umap.fit = test_umap_fit)
  
  
  names(scDesign3_umap_list) <- model_name
  scDesign3_umap_tbl <- bind_rows(scDesign3_umap_list, .id = "Method")
  
  scDesign3_umap_tbl <- scDesign3_umap_tbl %>% dplyr::mutate(Method = factor(Method, levels = model_name))
  
  umap_tbl <- bind_rows(test_umap_tbl, scDesign3_umap_tbl) %>% dplyr::mutate(Method = factor(Method, levels = c("test_data", model_name)))
  umap_tbl_list <- umap_tbl %>% group_by(Method) %>% group_split()
  # umap_lisi_label <- sapply(umap_tbl_list, function(x) {
  #   tbl <- dplyr::bind_rows(train_umap_tbl, x)
  #   
  #   tryCatch(expr = {
  #     lisi_fit <- lisi::compute_lisi(tbl[, c("UMAP1", "UMAP2")], meta_data = tbl[, "Method"], label_colnames = "Method")
  #     lisi_fit %>% unlist() %>% mean() %>% round(4)
  #   },
  #   error = function(error) {NA})
  # })
  
  rd_tbl <- dplyr::left_join(pca_tbl, umap_tbl)
  
  ## Construct sce list
  SingleCellExperiment::logcounts(train_sce) <- log1p(SingleCellExperiment::counts(train_sce))
  SummarizedExperiment::colData(train_sce)[, "Data"] <- "Train"
  SingleCellExperiment::logcounts(test_sce) <- log1p(SingleCellExperiment::counts(test_sce))
  SummarizedExperiment::colData(test_sce)$Data <- "Test"
  
  SingleCellExperiment::reducedDim(train_sce, "PCA") <- (as.matrix(train_pca_tbl %>% dplyr::select(dplyr::starts_with("PC"))))
  SingleCellExperiment::reducedDim(test_sce, "PCA") <- (as.matrix(test_pca_tbl %>% dplyr::select(dplyr::starts_with("PC"))))
  SingleCellExperiment::reducedDim(train_sce, "UMAP") <- (as.matrix(train_umap_tbl %>% dplyr::select(dplyr::starts_with("UMAP"))))
  SingleCellExperiment::reducedDim(test_sce, "UMAP") <- (as.matrix(test_umap_tbl %>% dplyr::select(dplyr::starts_with("UMAP"))))
  
  scDesign3_sce_list <- lapply(seq_len(length(model_name)), function(i) {
    sce <- train_sce
    SingleCellExperiment::counts(sce) <- scDesign3_count_list[[i]]
    SingleCellExperiment::logcounts(sce) <- log1p(SingleCellExperiment::counts(sce))
    
    SingleCellExperiment::reducedDim(sce, "PCA") <- (as.matrix(scDesign3_pca_list[[i]] %>% dplyr::select(dplyr::starts_with("PC"))))
    SingleCellExperiment::reducedDim(sce, "UMAP") <- (as.matrix(scDesign3_umap_list[[i]] %>% dplyr::select(dplyr::starts_with("UMAP"))))
    sce
  })
  
  sce_list_all <- append(list(train_sce), scDesign3_sce_list)
  ## Use CellMixS
  metric_res <- pbmcapply::pbmclapply(sce_list_all, function(x) {
    sce_combine <- cbind(test_sce, x)
    
    sce_pca <- CellMixS::evalIntegration(metrics = c("isi", "entropy", "cms"), sce_combine, 
                              k = 50, n_dim = 2, cell_min = 4,
                              res_name = c("weighted_isi", "entropy", "cms"), group = "Data", dim_red = "PCA")
    sce_umap <- CellMixS::evalIntegration(metrics = c("isi", "entropy", "cms"), sce_combine, 
                                         k = 50, n_dim = 2, cell_min = 4,
                                         res_name = c("weighted_isi", "entropy", "cms"), group = "Data", dim_red = "UMAP")
    PCA_lisi <- mean(colData(sce_pca)$weighted_isi)
    PCA_entropy <- mean(colData(sce_pca)$entropy)
    PCA_cms <- ks.test(colData(sce_pca)$cms.cms,"punif", 0, 1)$statistic
    
    UMAP_lisi <- mean(colData(sce_umap)$weighted_isi)
    UMAP_entropy <- mean(colData(sce_umap)$entropy)
    UMAP_cms <- ks.test(colData(sce_pca)$cms.cms,"punif", 0, 1)$statistic
    
    res <- c(PCA_lisi, PCA_entropy, PCA_cms, UMAP_lisi, UMAP_entropy, UMAP_cms)
    names(res) <- c("PCA_lisi", "PCA_entropy", "PCA_cms", "UMAP_lisi", "UMAP_entropy", "UMAP_cms")
    return(res)
    }, mc.cores = 20)
  metric_res <- simplify2array(metric_res)
  
  metric_tbl <- t(metric_res) %>% as_tibble() %>% dplyr::mutate(Method = c("test_data", as.character(model_name)), Marginal_AIC = aic_mat[, 1], Copula_AIC = aic_mat[, 2], Marginal_BIC = bic_mat[, 1], Copula_BIC = bic_mat[, 2])
  
  #metric_tbl <- tibble(Method = c("test_data", model_name), PCA_lisi = pca_lisi_label, UMAP_lisi = umap_lisi_label, Marginal_AIC = aic_mat[, 1], Copula_AIC = aic_mat[, 2], Marginal_BIC = bic_mat[, 1], Copula_BIC = bic_mat[, 2])
  
  ## UMAP plot
  # p_umap <- rd_tbl  %>% ggplot(aes(x = UMAP1, y = UMAP2, color = cell_type))+ geom_point(size = 0.5, alpha = 0.5) + facet_wrap(~Method, ncol = 5, labeller = label_parsed) + theme(legend.position = "bottom", aspect.ratio = 1)+ scale_colour_brewer(palette = "Set1") +theme_bw()
  # print(p_umap)
  # 
  # label_dat <- tibble(label = c(pca_lisi_label), Method = c("test_data", model_name), x = c(-20), y = c(30), col = "black")
  # 
  # p_pca <- rd_tbl  %>% ggplot(aes(x = PC1, y = PC2, color = cell_type))+ geom_point(size = 0.5, alpha = 0.5) + facet_wrap(~Method, ncol = 5, labeller = label_parsed) + 
  # theme(legend.position = "bottom", aspect.ratio = 1)+ scale_colour_brewer(palette = "Set1") +theme_bw()
  # print(p_pca)
  
  ## Add 6 metrics: 
  #1. mean of log gene 2. var of log gene 3. gene detection frequency
  #4. gene to gene cor 5. log library size 6.  cell detection frequency
  sce_list_metric <- append(list(test_sce), scDesign3_sce_list)
  
  mean_log_expr <- lapply(sce_list_metric, function(x) {
    MatrixGenerics::rowMeans2(SingleCellExperiment::logcounts(x))
  })
  var_log_expr <- lapply(sce_list_metric, function(x) {
    MatrixGenerics::rowVars(SingleCellExperiment::logcounts(x))
  })
  gene_detect_freq <- lapply(sce_list_metric, function(x) {
    MatrixGenerics::rowMeans2(SingleCellExperiment::counts(x) == 0)
  })
  gene_cor <- lapply(sce_list_metric, function(x) {
    cor_mat <- Rfast::cora(SingleCellExperiment::logcounts(x))
    cor_mat <- cor_mat[upper.tri(cor_mat)]
    cor_vec <- as.vector(cor_mat)
    cor_vec
  })
  log_lib_size <- lapply(sce_list_metric, function(x) {
    lib_size <- log(colSums2(SingleCellExperiment::counts(x)))
    lib_size
  })
  cell_detect_freq <- lapply(sce_list_metric, function(x) {
    MatrixGenerics::colMeans2(SingleCellExperiment::counts(x) == 0)
  })
  
  metric_tbl <- metric_tbl %>% dplyr::mutate(`Mean log expression` = mean_log_expr,
                                             `Var log expression` = var_log_expr,
                                             `Gene detection freq` = gene_detect_freq,
                                             `Gene correlation` = gene_cor,
                                             `Log library size` = log_lib_size,
                                             `Cell detection freq` = cell_detect_freq,) 
  
  ## Create corr mat list
  ngene_cor <- min(ngene_cor, dim(SingleCellExperiment::logcounts(sce_list_all[[1]]))[1])
  corr_mat_test <- Rfast::cora(SingleCellExperiment::logcounts(sce_list_all[[1]])[1:ngene_cor,])
  
  hclust_res <- hclust(dist(corr_mat_test))
  corr_mat_list <-  pbmcapply::pbmclapply(sce_list_all, function(x) {
    corr_mat <- Rfast::cora(SingleCellExperiment::logcounts(x)[1:ngene_cor,])
    corr_mat <- corr_mat[hclust_res$order, hclust_res$order]
  }, mc.cores = 10)
  new_model_name <- c("test_data", as.character(model_name))

  names(corr_mat_list) <- new_model_name
  cor_melted <- lapply(corr_mat_list, reshape2::melt)
  cor_dat <- Reduce(rbind, cor_melted)
  cor_dat$Method <- Reduce(c, lapply(new_model_name, function(x){
    rep(x, nrow(cor_melted[[x]]))
  }))
  cor_dat$Method <- factor(cor_dat$Method, levels = new_model_name)
  #cor_dat$method = mapvalues(cor_dat$method,
  #                           from = method_list, to = method_name_list)
  
  return(list(rd_tbl = rd_tbl, metric_tbl = metric_tbl, corr_dat = cor_dat))
  
}

extra_cdi <- function(res, model_name){
  ## Extract AIC
  if(is.null(res[[1]]$CDI_AIC)) {
    aic_mat <- sapply(res, function(x) x$CDI_AIC)
  } else aic_mat <- sapply(res, function(x) x$CDI_AIC)
  
  # colnames(aic_mat) <- "CDI_AIC"
  # rownames(aic_mat) <- model_name
  
  ## Extract bic
  if(is.null(res[[1]]$CDI_BIC)) {
    bic_mat <- sapply(res, function(x) x$CDI_BIC)
  } else bic_mat <- sapply(res, function(x) x$CDI_BIC)
  # colnames(bic_mat) <- "CDI_BIC"
  # rownames(bic_mat) <- model_name
  
  dat <- data.frame(CDI_AIC = aic_mat, CDI_BIC = bic_mat, Method = model_name)
  return(dat)
  
}
