require(Rfast)

createSummary <- function(train_sce, 
                          test_sce, 
                          scDesign3Vine_result,
                          formula_mat,
                          other_sce_list,
                          other_sce_name,
                          tfidf = FALSE,
                          ngene_cor = 100,
                          ngene_pca = NULL,
                          n_neighbors = 15, 
                          min_dist = 0.1,
                          normalization = FALSE,
                          gene_vec = NULL) {
  if(is.vector(formula_mat)) {
    model_name <- formula_mat
  } else {
    model_name <- apply(formula_mat, 1, function(x) {
      paste0(x, collapse = "|")
    })
  }
  
  ##
  test_sce <- test_sce[, colSums(counts(test_sce)) > 0]
  
  ## Extract AIC
  if(is.null(scDesign3Vine_result[[1]]$model_aic)) {
    aic_mat <- t(sapply(scDesign3Vine_result, function(x) x$model.aic))
  } else aic_mat <- t(sapply(scDesign3Vine_result, function(x) x$model_aic))
  
  colnames(aic_mat) <- c("Marginal_AIC", "Copula_AIC", "Total_AIC")
  rownames(aic_mat) <- model_name
  ref_mat <- matrix(NA, 2, 3)
  colnames(ref_mat) <- c("Marginal_AIC", "Copula_AIC", "Total_AIC")
  rownames(ref_mat) <- c("Train", "Test")
  other_mat <- matrix(NA, length(other_sce_name), 3)
  colnames(other_mat) <- c("Marginal_AIC", "Copula_AIC", "Total_AIC")
  rownames(other_mat) <- other_sce_name
  aic_mat <- rbind(ref_mat, aic_mat, other_mat)
  
  ## Extract bic
  if(is.null(scDesign3Vine_result[[1]]$model_bic)) {
    bic_mat <- t(sapply(scDesign3Vine_result, function(x) x$model.bic))
  } else bic_mat <- t(sapply(scDesign3Vine_result, function(x) x$model_bic))
  colnames(bic_mat) <- c("Marginal_BIC", "Copula_BIC", "Total_BIC")
  rownames(bic_mat) <- model_name
  ref_mat <- matrix(NA, 2, 3)
  colnames(ref_mat) <- c("Marginal_BIC", "Copula_BIC", "Total_BIC")
  rownames(ref_mat) <- c("Train", "Test")
  other_mat <- matrix(NA, length(other_sce_name), 3)
  colnames(other_mat) <- c("Marginal_AIC", "Copula_AIC", "Total_AIC")
  rownames(other_mat) <- other_sce_name
  bic_mat <- rbind(ref_mat, bic_mat, other_mat)
  
  print(cbind(aic_mat, bic_mat))
  
  ## Create scDesign3 sce list
  if(is.null(scDesign3Vine_result[[1]]$new_count)) {
    scDesign3_count_list <- lapply(scDesign3Vine_result, function(x) x$new.count)
  } else scDesign3_count_list <- lapply(scDesign3Vine_result, function(x) x$new_count)
  
  
  scDesign3_sce_list <- lapply(seq_len(length(model_name)), function(i) {
    print(model_name[i])
    sce <- train_sce
    #sce <- SingleCellExperiment::SingleCellExperiment(list(counts = scDesign3_count_list[[i]]))
    colnames(scDesign3_count_list[[i]]) <- colnames(train_sce)
    SingleCellExperiment::counts(sce) <- scDesign3_count_list[[i]]
    SingleCellExperiment::logcounts(sce) <- log1p(SingleCellExperiment::counts(sce))
    colData(sce) <- colData(train_sce)
    sce <- sce[, colSums(counts(sce)) > 0]
    sce
  })
  
  other_sce_list <- lapply(other_sce_list, function(sce) {
    sce <- sce[, colSums(counts(sce)) > 0]
    sce
  })
  
  sce_list <- c(list(train_sce, test_sce), scDesign3_sce_list, other_sce_list)
  sce_list_name <- c("train", "test", model_name, other_sce_name)
  
  ## Construct PCA
  if(!tfidf) {
    if(normalization) {
      test_sce <- scuttle::logNormCounts(test_sce, size.factors = NULL)
      #test_pca_fit <- irlba::prcomp_irlba(t(log1p(scale(counts(test_sce), center=FALSE, scale=colSums(counts(test_sce)))*20000)), 
      #                                    center = TRUE, 
      #                                    scale. = TRUE, 
      #                                    n = 20)
      test_pca_fit <- irlba::prcomp_irlba(t(logcounts(test_sce)),
                                         center = TRUE,
                                         scale. = FALSE,
                                         n = 50)
      
      sce_list <- lapply(seq_len(length(sce_list)), function(x) {
        print(x)
        x <- sce_list[[x]]
        counts(x) <- as.matrix(counts(x))
        x <- scuttle::logNormCounts(x)
        #SingleCellExperiment::reducedDim(x, "PCA") <- predict(test_pca_fit, newdata= t(log1p(scale(counts(x), center=FALSE, scale=colSums(counts(x)))*20000)))
        SingleCellExperiment::reducedDim(x, "PCA") <- predict(test_pca_fit, newdata= t(logcounts(x)))
        return(x)
      })
    }
    else {
      test_pca_fit <- irlba::prcomp_irlba(t(log1p(counts(test_sce))), 
                                          center = TRUE, 
                                          scale. = FALSE, 
                                          n = 50)
      
      sce_list <- lapply(seq_len(length(sce_list)), function(x) {
        print(x)
        x <- sce_list[[x]]
        counts(x) <- as.matrix(counts(x))
        logcounts(x) <- log1p(counts(x))
        SingleCellExperiment::logcounts(x) <- log1p(SingleCellExperiment::counts(x))
        SingleCellExperiment::reducedDim(x, "PCA") <- predict(test_pca_fit, newdata= t(logcounts(x)))
        return(x)
      })
    }
  } else {
    test_pca_fit <- irlba::prcomp_irlba(t(log1p(Signac::RunTFIDF(counts(test_sce)))), 
                                        center = TRUE, 
                                        scale. = TRUE, 
                                        n = 50)
    
    sce_list <- lapply(seq_len(length(sce_list)), function(x) {
      print(x)
      x <- sce_list[[x]]
      counts(x) <- as.matrix(counts(x))
      logcounts(x) <- log1p(counts(x))
      SingleCellExperiment::logcounts(x) <- log1p(SingleCellExperiment::counts(x))
      SingleCellExperiment::reducedDim(x, "PCA") <- predict(test_pca_fit, newdata= t(log1p(Signac::RunTFIDF(counts(x)))))
      return(x)
    })
  }
  
  
  print("PCA finished!")
  
  ## UMAP
  test_umap_fit <- umap::umap(test_pca_fit$x, n_neighbors = n_neighbors, min_dist = min_dist)
  
  sce_list <- pbmcapply::pbmclapply(sce_list, function(x) {
    res <- predict(object = test_umap_fit, data= (reducedDim(x, "PCA")))
    colnames(res) <- c("UMAP1", "UMAP2")
    SingleCellExperiment::reducedDim(x, "UMAP") <- res
    return(x)
  }, mc.cores = 20)
  print("UMAP finished!")
  
  sce_list <- lapply(seq_len(length(sce_list)), function(x) {
    sce <- sce_list[[x]]
    colData(sce)$Method <-  sce_list_name[x]
    sce
  })
  names(sce_list) <- sce_list_name
  ## Extract rd
  rd_list <- lapply(sce_list, function(x) {
    rd <- colData(x) %>% as_tibble()
    rd_pca <- SingleCellExperiment::reducedDim(x, "PCA") %>% as_tibble()
    rd_umap <- SingleCellExperiment::reducedDim(x, "UMAP") %>% as_tibble()
    
    rd %>% bind_cols(rd_pca) %>% bind_cols(rd_umap)
  })
  
  names(rd_list) <- names(sce_list)
  
  rd_tbl <- bind_rows(rd_list, .id = "Method")
  
  print("PCA and UMAP finished!")
  
  print("Start CellMixS")
  ## Use CellMixS 
  ## pbmcapply::pbmc
  test_sce2 <- sce_list[[2]]
  colData(test_sce2)$Method <- "test2"
  metric_res <- pbmcapply::pbmclapply(sce_list, function(x) {
    colnames(x) <- paste0("Cell", seq_len(dim(x)[2]))
    count_combine <- cbind(counts(x), counts(test_sce2))
    sce_combine <- SingleCellExperiment(list(counts = count_combine))
    reducedDim(sce_combine, "PCA") <- rbind(reducedDim(x, "PCA"), reducedDim(test_sce2, "PCA"))
    reducedDim(sce_combine, "UMAP") <- rbind(reducedDim(x, "UMAP"), reducedDim(test_sce2, "UMAP"))
    colData(sce_combine)$Method <- c(colData(x)$Method, colData(test_sce2)$Method)
    
    
    sce_pca <- CellMixS::evalIntegration(metrics = c("isi", "entropy", "cms"), sce_combine, 
                                         k = 50, n_dim = 2, cell_min = 4,
                                         res_name = c("weighted_isi", "entropy", "cms"), group = "Method", dim_red = "PCA")
    sce_umap <- CellMixS::evalIntegration(metrics = c("isi", "entropy", "cms"), sce_combine, 
                                          k = 50, n_dim = 2, cell_min = 4,
                                          res_name = c("weighted_isi", "entropy", "cms"), group = "Method", dim_red = "UMAP")
    PCA_lisi <- mean(colData(sce_pca)$weighted_isi)
    PCA_entropy <- mean(colData(sce_pca)$entropy)
    PCA_cms <- ks.test(colData(sce_pca)$cms.cms,"punif", 0, 1)$statistic
    
    UMAP_lisi <- mean(colData(sce_umap)$weighted_isi)
    UMAP_entropy <- mean(colData(sce_umap)$entropy)
    UMAP_cms <- ks.test(colData(sce_pca)$cms.cms,"punif", 0, 1)$statistic
    
    res <- c(PCA_lisi, PCA_entropy, PCA_cms, UMAP_lisi, UMAP_entropy, UMAP_cms)
    names(res) <- c("PCA_lisi", "PCA_entropy", "PCA_cms", "UMAP_lisi", "UMAP_entropy", "UMAP_cms")
    return(res)
  }, mc.cores = 20)
  metric_res <- simplify2array(metric_res)
  
  metric_tbl <- t(metric_res) %>% as_tibble() %>% dplyr::mutate(Method = sce_list_name, Marginal_AIC = aic_mat[, 1], Copula_AIC = aic_mat[, 2], Marginal_BIC = bic_mat[, 1], Copula_BIC = bic_mat[, 2])
  
  print("End CellMixS")
  
  ## Add 6 metrics: 
  #1. mean of log gene 2. var of log gene 3. gene detection frequency
  #4. gene to gene cor 5. log library size 6.  cell detection frequency

  mean_log_expr <- lapply(sce_list, function(x) {
    MatrixGenerics::rowMeans2(SingleCellExperiment::logcounts(x))
  })
  var_log_expr <- lapply(sce_list, function(x) {
    MatrixGenerics::rowVars(SingleCellExperiment::logcounts(x))
  })
  gene_detect_freq <- lapply(sce_list, function(x) {
    MatrixGenerics::rowMeans2(SingleCellExperiment::counts(x) == 0)
  })
  gene_cor <- lapply(sce_list, function(x) {
    cor_mat <- Rfast::cora(SingleCellExperiment::logcounts(x))
    cor_mat <- cor_mat[upper.tri(cor_mat)]
    cor_vec <- as.vector(cor_mat)
    cor_vec
  })
  log_lib_size <- lapply(sce_list, function(x) {
    lib_size <- log(colSums2(SingleCellExperiment::counts(x)))
    lib_size
  })
  cell_detect_freq <- lapply(sce_list, function(x) {
    MatrixGenerics::colMeans2(SingleCellExperiment::counts(x) == 0)
  })
  
  metric_tbl <- metric_tbl %>% dplyr::mutate(`Mean log expression` = mean_log_expr,
                                             `Var log expression` = var_log_expr,
                                             `Gene detection freq` = gene_detect_freq,
                                             `Gene correlation` = gene_cor,
                                             `Log library size` = log_lib_size,
                                             `Cell detection freq` = cell_detect_freq,) 
  
  ## Create corr mat list
  train_logcounts <- t(as.matrix(SingleCellExperiment::logcounts(sce_list[[1]])))
  ngene_cor <- min(ngene_cor, dim(train_logcounts)[2])
  feature_index <- order(colSums(train_logcounts), decreasing = TRUE)
  
  if(is.null(gene_vec)) {
    corr_mat_test <- cor(train_logcounts[, feature_index <= ngene_cor], use = "pairwise.complete")
  }
  else {
    corr_mat_test <- cor(train_logcounts[, gene_vec], use = "pairwise.complete")
  }
  
  hclust_res <- hclust(dist(corr_mat_test))
  corr_mat_list <- pbmcapply::pbmclapply(sce_list, function(x) {
    if(is.null(gene_vec)) {
    mat <- as.matrix(t(SingleCellExperiment::logcounts(x)[feature_index <= ngene_cor,]))}
    else {
      mat <- as.matrix(t(SingleCellExperiment::logcounts(x)[gene_vec,]))  
    }
    corr_mat <- Rfast::cora(mat)
    corr_mat <- corr_mat[hclust_res$order, hclust_res$order]
    #diag(corr_mat) <- NA
    corr_mat
  }, mc.cores = 20)
  
  names(corr_mat_list) <- sce_list_name
  cor_melted <- lapply(corr_mat_list, reshape2::melt)
  cor_dat <- Reduce(rbind, cor_melted)
  cor_dat$Method <- Reduce(c, lapply(sce_list_name, function(x){
    rep(x, nrow(cor_melted[[x]]))
  }))
  cor_dat$Method <- factor(cor_dat$Method, levels = sce_list_name)
  
  return(list(rd_tbl = rd_tbl, metric_tbl = metric_tbl, corr_dat = cor_dat))
  
}
