library(SingleCellExperiment)
library(useful)
library(irlba)
library(tidyverse)
library(scales)
library(umap)
library(lisi)
library(mgcv)
library(MASS)
library(pheatmap)
library(ComplexHeatmap)
library(mvtnorm)
library(Rfast)
library(mvnfast)
library(Matrix)
library(spcov)
library(umap)
library(circlize)
library(scater)
library(scran)
library(slingshot)
devtools::install_local("~/Package/scDesign3")
devtools::document("~/Package/scDesign3")

data_set <- c("EMBYRO", "PANCREAS", "GYRUS")
for(data in data_set){
  path <- paste0("~/scDesign3/data/Perturb/", data, "_500_train_sce.rds")
  train_sce <- readRDS(path)
  test_sce <- readRDS(paste0("~/scDesign3/data/Perturb/", data,"_500_test_sce.rds"))
  train_sce <- train_sce[-c(gene_zero,gene_zero2),]
  test_sce <- test_sce[-c(gene_zero,gene_zero2),]
  train_sce_list <- lapply(0:10, function(x) {
    set.seed(234)
    perturb_prop <- x/10
    n_cell <- round(dim(train_sce)[2]*perturb_prop)
    cell_index <- sample(1:dim(train_sce)[2], n_cell)

    new_pseudotime <- colData(train_sce)$pseudotime
    new_pseudotime[cell_index] <- runif(n_cell)

    curr_sce <- train_sce
    colData(curr_sce)$pseudotime <- new_pseudotime
    curr_sce
  })


  scDesign3_result <- lapply(train_sce_list, function(x) {
    print(Sys.time())
    res <-  scdesign3(
      sce = x,
      assay_use = "counts",
      celltype = NULL,
      pseudotime = "pseudotime",
      spatial = NULL,
      other_covariates = NULL,
      mu_formula = "s(pseudotime, bs = 'cr', k = 10)",
      sigma_formula = 1,
      corr_formula = "ind",
      copula = "vine",
      n_cores = 50
    )

    return(res)
  })
  saveRDS(scDesign3_result, paste0("~/scDesign3/data/perturb_for_paper/",data,"_scDesign3.rds"))
  for (i in 1:11) {
    x <- scDesign3_result[[i]]
    new_sce <- SingleCellExperiment(list(counts = x$new_count), colData = colData(train_sce_list[[i]]))
    path <- paste0("~/scDesign3/data/", data ,"_pseudotime_perturb/cor_ind/")
    curr_name <- paste0("EMBYRO_preturb_",(i-1)*10,"_sce.rds")
    full_path<-paste0(path, curr_name)
    saveRDS(new_sce, full_path)
  }
}


data_set <- c("dyngen_linear_simple", "dyngen_linear_tf10", "dyngen_linear_simple_tf10","dyngen_linear_tf30","dyngen_linear")
for(data in data_set){
  path <- paste0("~/scDesign3/data/Perturb/", data, "_500_train_sce.rds")
  train_sce <- readRDS(path)
  test_sce <- readRDS(paste0("~/scDesign3/data/Perturb/", data,"_500_test_sce.rds"))
  train_sce <- train_sce[-c(gene_zero,gene_zero2),]
  test_sce <- test_sce[-c(gene_zero,gene_zero2),]
  train_sce_list <- lapply(0:10, function(x) {
    set.seed(234)
    perturb_prop <- x/10
    n_cell <- round(dim(train_sce)[2]*perturb_prop)
    cell_index <- sample(1:dim(train_sce)[2], n_cell)

    new_pseudotime <- colData(train_sce)$pseudotime
    new_pseudotime[cell_index] <- runif(n_cell)

    curr_sce <- train_sce
    colData(curr_sce)$pseudotime <- new_pseudotime
    curr_sce
  })


  scDesign3_result <- lapply(train_sce_list, function(x) {
    print(Sys.time())
    res <-  scdesign3(
      sce = x,
      assay_use = "counts",
      celltype = NULL,
      pseudotime = "pseudotime",
      spatial = NULL,
      other_covariates = NULL,
      mu_formula = "s(pseudotime, bs = 'cr', k = 10)",
      sigma_formula = 1,
      corr_formula = "ind",
      copula = "vine",
      n_cores = 50
    )

    return(res)
  })
  saveRDS(scDesign3_result, paste0("~/scDesign3/data/perturb_for_paper/",data,"_scDesign3.rds"))
  R2<- sapply(1:11, function(x){
    unchanged <- colData(train_sce)$pseudotime
    new_pseudotime <- colData(train_sce_list[[x]])$pseudotime
    r2 <- cor(unchanged, new_pseudotime)^2
    r2
  })
  name <- paste0(data,"_",seq(0,100,10),"%")
  res <- createSummary(train_sce, test_sce, scDesign3_result, name)
  met <- res$metric_tbl %>% subset(select = c("PCA_lisi","UMAP_lisi","Marginal_AIC","Marginal_BIC","Method")) %>% filter(Method != "test_data")
  met$R2 <- R2
  saveRDS(met, paste0("~/scDesign3/data/perturbDyngen/res/",data,"_metric.rds"))
}
