library(scDesign3)
library(tidyverse)
library(igraph)
library(scran)
library(rvinecopulib)
library(irlba)
library(lisi)
library(cowplot)
library(pracma)
library(umap)
library(ggpubr)
library(ggrepel)
library(CellMixS)
library(ggpubr)
source("~/scDesign3/code/Performance0415.R")
source("~/scDesign3/code/Clustering.R")
suppressPackageStartupMessages({
  library(DuoClustering2018)
})
library(CDI)


name_vec <- c("Koh", "Kumar", "SimKumar4easy", "SimKumar4hard", "SimKumar8hard", "Zhengmix4eq", "Zhengmix4uneq", "Zhengmix8eq")

for (dataname in name_vec) {
  dataname <- "Zhengmix8eq"
  if(dir.exists(paste0("~/scDesign3/data/clustering/", dataname, "/"))) {
    message(paste0("Folder ", dataname, " exists"))
  } else {
    dir.create(paste0("~/scDesign3/data/clustering/", dataname, "/"))
  }

  res <- get(paste0("clustering_summary_filteredExpr10_", dataname, "_v1"))(metadata = FALSE)

  res_sub <- res %>% dplyr::filter(method %in% c("SC3", "Seurat", "PCAHC", "PCAKmeans", "CIDR") & run == 1)
  res_sub_list <- res_sub %>% group_by(dataset, method, run, k, resolution) %>% group_split()


  ## This will create a tibble. Each row is a clustering label with ARI.
  res_summary <- res.summary(res_sub) %>% tibble::rowid_to_column( "index")

  nonred_index <- res_summary %>% dplyr::group_by(ARI) %>% dplyr::mutate(rank = row_number()) %>% dplyr::filter(rank == 1) %>% .$index

  res_sub_list <- res_sub_list[nonred_index]
  res_summary <- res_summary[nonred_index, ]

  sce <- get(paste0("sce_filteredExpr10_", dataname))(metadata = FALSE)


  ncell <- ncol(sce)
  ngene <- 500

  chosen <- getTopHVGs(sce, n = ngene)
  sce <- sce[chosen, ]
  ntrain <- round(ncell/2)

  set.seed(123)
  train_index <- sample(seq_len(ncell), ntrain, replace = FALSE)

  train_sce <- sce[, train_index]
  test_sce <- sce[, -train_index]


  if(file.exists(paste0("~/scDesign3/data/clustering/", dataname, "/CDI_train_sce.rds"))) {
    train_sce <- readRDS(paste0("~/scDesign3/data/clustering/", dataname, "/CDI_train_sce.rds"))
  } else {
    saveRDS(train_sce, paste0("~/scDesign3/data/clustering/", dataname, "/CDI_train_sce.rds"))
  }

  if(file.exists(paste0("~/scDesign3/data/clustering/", dataname, "/CDI_test_sce.rds"))) {
    test_sce <- readRDS(paste0("~/scDesign3/data/clustering/", dataname, "/CDI_test_sce.rds"))
  } else {
    saveRDS(test_sce, paste0("~/scDesign3/data/clustering/", dataname, "/CDI_test_sce.rds"))
  }


  if(file.exists(paste0("~/scDesign3/data/clustering/", dataname, "/CDI_selected_result.rds"))) {
    cdi_result <- readRDS(paste0("~/scDesign3/data/clustering/", dataname, "/CDI_selected_result.rds"))
    print(paste0(dataname, " do not need to run again"))
  } else {
    cdi_result <- lapply(res_sub_list, function(x) {
      dat <- x %>% dplyr::select(c("cell", "cluster")) %>% data.frame()
      rownames(dat) <- dat$cell
      colData(train_sce)$cell_type <- factor(dat[colnames(train_sce), ]$cluster)


      sf <- size_factor(counts(train_sce))
      simu_sce <- calculate_CDI(
        sub_gcmat = round(counts(train_sce),0),
        cand_lab_df = as.vector(colData(train_sce)$cell_type),
        cell_size_factor = as.vector(sf),
        ncore = 10)


      simu_sce
    })

    saveRDS(cdi_result
            , paste0("~/scDesign3/data/clustering/", dataname, "/CDI_selected_result.rds"))
    print(paste0(dataname, " done"))
    print(Sys.time())
  }
  model_name <- apply(res_summary %>% dplyr::select(c("filtering",       "dataset",       "method",   "run", "k",   "resolution", "ARI")), 1, function(x) {
    paste(x, sep = ", ", collapse = ", ")
  })

  cdi_res <- extra_cdi(cdi_result, model_name)
  cdi_res$ARI <- res_summary$ARI
  cdi_res$k <- res_summary$k
  saveRDS(cdi_res, paste0("~/scDesign3/data/clustering/", dataname, "/CDI_selected_metric.rds"))
  print(paste0(dataname, " eval done"))
  print(Sys.time())
}
