library(scDesign3)
library(tidyverse)
library(igraph)
library(scran)
library(rvinecopulib)
library(irlba)
library(lisi)
library(cowplot)
library(pracma)
library(umap)
library(ggpubr)
library(ggrepel)
library(CellMixS)
library(ggpubr)
source("~/scDesign3/code/Performance0415.R")
source("~/scDesign3/code/Clustering.R")
suppressPackageStartupMessages({
  library(DuoClustering2018)
})

name_vec <- c("Koh", "Kumar", "SimKumar4easy", "SimKumar4hard", "SimKumar8hard", "Zhengmix4eq", "Zhengmix4uneq", "Zhengmix8eq")

for (dataname in name_vec) {

  if(dir.exists(paste0("~/scDesign3/data/clustering/", dataname, "/"))) {
    message(paste0("Folder ", dataname, " exists"))
  } else {
    dir.create(paste0("~/scDesign3/data/clustering/", dataname, "/"))
  }

  res <- get(paste0("clustering_summary_filteredExpr10_", dataname, "_v1"))(metadata = FALSE)

  res_sub <- res %>% dplyr::filter(method %in% c("SC3", "Seurat", "PCAHC", "PCAKmeans", "CIDR") & run == 1)
  res_sub_list <- res_sub %>% group_by(dataset, method, run, k, resolution) %>% group_split()

  ## This will create a tibble. Each row is a clustering label with ARI.
  res_summary <- res.summary(res_sub) %>% tibble::rowid_to_column( "index")

  nonred_index <- res_summary %>% dplyr::group_by(ARI) %>% dplyr::mutate(rank = row_number()) %>% dplyr::filter(rank == 1) %>% .$index

  res_sub_list <- res_sub_list[nonred_index]
  res_summary <- res_summary[nonred_index, ]

  sce <- get(paste0("sce_filteredExpr10_", dataname))(metadata = FALSE)

  ncell <- ncol(sce)
  ngene <- 500

  chosen <- getTopHVGs(sce, n = ngene)
  sce <- sce[chosen, ]
  ntrain <- round(ncell/2)

  set.seed(123)
  train_index <- sample(seq_len(ncell), ntrain, replace = FALSE)

  train_sce <- sce[, train_index]
  test_sce <- sce[, -train_index]

  if(file.exists(paste0("~/scDesign3/data/clustering/", dataname, "/train_sce.rds"))) {
    train_sce <- readRDS(paste0("~/scDesign3/data/clustering/", dataname, "/train_sce.rds"))
  } else {
    saveRDS(train_sce, paste0("~/scDesign3/data/clustering/", dataname, "/train_sce.rds"))
  }

  if(file.exists(paste0("~/scDesign3/data/clustering/", dataname, "/test_sce.rds"))) {
    test_sce <- readRDS(paste0("~/scDesign3/data/clustering/", dataname, "/test_sce.rds"))
  } else {
    saveRDS(test_sce, paste0("~/scDesign3/data/clustering/", dataname, "/test_sce.rds"))
  }

  if(file.exists(paste0("~/scDesign3/data/clustering/", dataname, "/scDesign3Vine_result.rds"))) {
    scDesign3_result <- readRDS(paste0("~/scDesign3/data/clustering/", dataname, "/scDesign3Vine_result.rds"))
  } else {
    scDesign3_result <- lapply(res_sub_list, function(x) {
      dat <- x %>% dplyr::select(c("cell", "cluster")) %>% data.frame()
      #rownames(dat) <- NULL
      rownames(dat) <- dat$cell
      colData(train_sce)$cell_type <- factor(dat[colnames(train_sce), ]$cluster)

      simu_sce <- scdesign3(sce = train_sce,
                            celltype = 'cell_type',
                            pseudotime = NULL,
                            spatial = NULL,
                            other_covariates = NULL,
                            corr_formula = "1",
                            mu_formula = "cell_type",
                            sigma_formula = "cell_type",
                            n_cores = 50,
                            copula = "vine",
                            assay_use = "counts",
                            family_use = "nb")

      simu_sce
    })

    saveRDS(scDesign3_result
            , paste0("~/scDesign3/data/clustering/", dataname, "/scDesign3Vine_result.rds"))
  }
  model_name <- apply(res_summary %>% dplyr::select(c("filtering",       "dataset",       "method",   "run", "k",   "resolution", "ARI")), 1, function(x) {
    paste(x, sep = ", ", collapse = ", ")
  })

  scDesign3_res <- createSummary(train_sce = train_sce, test_sce = test_sce, scDesign3Vine_result = scDesign3_result, formula_mat = model_name)

  scDesign3_res$metric_tbl$ARI <- c(NA, res_summary$ARI)
  scDesign3_res$metric_tbl$AMI <- c(NA, res_summary$AMI)
  scDesign3_res$metric_tbl$NMI <- c(NA, res_summary$NMI)
  scDesign3_res$metric_tbl$clustering_method <- c(NA, res_summary$method)
  scDesign3_res$metric_tbl$k <- c(NA, res_summary$k)
  saveRDS(scDesign3_res$metric_tbl, paste0("~/scDesign3/data/clustering/", dataname, "/metric_tbl.rds"))
  p <- scDesign3_res$metric_tbl %>% ggplot(aes(x = ARI, y = Marginal_BIC)) + geom_point() + ggtitle(dataname) + theme_bw() + stat_cor(method = "spearman")  + theme(aspect.ratio = 1)
  plot(p)
  cowplot::ggsave2(filename = paste0("~/scDesign3/data/clustering/", dataname, "/ARI_AIC.png"), plot = p, device = "png")
  gc()
}
